<?php
/**
 * YANA Commandline interface
 *
 * This simple program is used to trigger the YANA framework from
 * the command line, for example via a cron-job.
 *
 * To run this script use the following syntax:
 * php cli.php "arg1=value 1" "arg2=value 2" ...
 *
 * Note: errors are printed to a log file (cache/error.log).
 *
 * Note: as of the web-interface the argument "action" is mandatory.
 * The arguments are lower-cased before being processed.
 *
 *
 * Software:        Yana PHP-Framework 3.0.0
 * Author:          Dipl. Inf. Thomas Meyer
 * Homepage:        http://www.yanaframework.net
 * Copyright:       (c) 2009  Thomas Meyer
 * License:         GNU GPL  http://www.gnu.org/licenses/
 *
 * This program: can be redistributed and/or modified under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/.
 *
 * You may use this software for free. In case you like it, or want to
 * use it for a commercial purpose, please consider making a donation.
 * http://yanaframework.net/donate.php
 *
 * This notice MAY NOT be removed.
 *
 *
 * @package  yana
 * @license  http://www.gnu.org/licenses/gpl.txt
 */

/**#@+
 * @ignore
 */
require_once('library.php');
/**#@-*/

/* prepare input arguments */
$argv = $_SERVER['argv'];
array_shift($argv);

/** BIRTH: $_argv
 * temporary var of type array
 */
$_argv = array();
/** BIRTH: $_arg
 * temporary var of type string
 */
$_arg = "";

while (!empty($argv))
{
    $_arg = array_shift($argv);
    /** BIRTH: $m */
    if (preg_match('/^(\w[\w\/-_]*)=(.*)/', $_arg, $m)) {
        $_argv[strtolower($m[1])] = $m[2];
    } else {
        trigger_error("Malformed argument: \"${arg}\". Expected syntax \"arg1=value 1\".", E_USER_ERROR);
        exit(1);
    }
    /** EOL: $m */
    unset($m);
}
$argv = $_argv;

/** EOL: $_arg */
unset($_arg);
/** EOL: $_argv */
unset($_argv);

if (!isset($argv['action'])) {
    trigger_error("Missing argument: \"action\". This argument is mandatory. Syntax: \"action=foo\"", E_USER_ERROR);
    exit(1);
}

/* Initialize the framework */
ErrorUtility::setErrorReporting(YANA_ERROR_LOG);
$YANA = new Yana("config/system.config", $argv);

/* Output a standard header */
print "
---------------------------------
| Yana - command line interface |
---------------------------------

Running:  ".$_SERVER['argv'][0]."
Time:     ".date('r')."
Arguments:
".print_r($argv, true)."\n";

$YANA->handle($argv['action'], $argv);
foreach ($YANA->log as $log)
{
    switch (strtolower(get_class($log)))
    {
        case 'log':  case 'message':
            trigger_error($log->getMessage(), E_USER_NOTICE);
        break;
        case 'alert':
            trigger_error($log->getMessage(), E_USER_NOTICE);
        break;
        case 'warning':
            trigger_error($log->getMessage(), E_USER_WARNING);
        break;
        case 'error':
            trigger_error($log->getMessage(), E_USER_ERROR);
        break;
        default:
            trigger_error($log->getMessage(), E_USER_WARNING);
        break;
    }
}

if ($YANA->getVar('SETTINGS.LAST_RESULT') === true) {
    print 'Execution finished successfully.';
    exit(0);
} else {
    print 'Execution finished with errors. See the error log for details. (default output path is: cache/error.log)';
    trigger_error('Execution finished with errors.', E_USER_WARNING);
    exit(2);
}
// Copyright  2009  Thomas Meyer  http://www.yanaframework.net
?>