<?php
/**
 * YANA library
 *
 *
 * Software:        Yana PHP-Framework 3.0.0
 * Author:          Dipl. Inf. Thomas Meyer
 * Homepage:        http://www.yanaframework.net
 * Copyright:       (c) 2009  Thomas Meyer
 * License:         GNU GPL  http://www.gnu.org/licenses/
 *
 * This program: can be redistributed and/or modified under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/.
 *
 * You may use this software for free. In case you like it, or want to
 * use it for a commercial purpose, please consider making a donation.
 * http://yanaframework.net/donate.php
 *
 * This notice MAY NOT be removed.
 *
 *
 * @package  yana
 * @license  http://www.gnu.org/licenses/gpl.txt
 */

/**
 * decorator  database Updater
 *
 * This decorator class is intended to create compare the structure of an existing
 * database with a given structure file.
 *
 * @access      public
 * @package     yana
 * @subpackage  database
 * @since       2.9.7
 * @ignore
 */
class DbUpdater extends DbCreator
{
    /**#@+
     * @ignore
     * @access  private
     */

    /**
     * MYSQL syntax
     *
     * @var  array
     */
    var $mysql = array(
        'rename' => array(
            'table' => 'RENAME TABLE `' . YANA_DATABASE_PREFIX . '%TABLE%` TO `' . YANA_DATABASE_PREFIX . '%NEW_NAME%`';
            'column' => 'ALTER TABLE `' . YANA_DATABASE_PREFIX . '%TABLE%` CHANGE `%COLUMN%` `%NEW_NAME%`';
        ),
        'drop' => array(
            'table' => 'DROP TABLE IF EXISTS `%' . YANA_DATABASE_PREFIX . 'TABLE%`';
            'column' => 'ALTER TABLE `%' . YANA_DATABASE_PREFIX . 'TABLE%` DROP `%COLUMN%`';
            'index' => 'DROP INDEX `%INDEX%` ON `%TABLE%`';
        ),
        'update' => array(
            'table' => 'ALTER TABLE `%' . YANA_DATABASE_PREFIX . 'TABLE%` %ALTER_SPECIFICATION%';
            'column' => 'MODIFY `%COLUMN%` %DEFINITION%';
        ),
        'create' => array(
            'table' => 'CREATE TABLE IF NOT EXISTS `%' . YANA_DATABASE_PREFIX . 'TABLE%` %ALTER_SPECIFICATION%';
            'column' => '';
        )
    );

    /**#@-*/

    /**
     * compare database with structure file
     *
     * Returns a list of differences or bool(false) on error.
     *
     * @access  public
     * @param   DbStream     $db         source database
     * @param   DbStructure  $structure  target structure
     * @return  array
     * @since   2.9.7
     */
    function compareDatabase($db, $structure)
    {
        /**
         * check input arguments
         */
        if (!is_object($db) || !(is_a($db, "FileDb") || is_a($db, "DbStream"))) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'Instance of DbStream', gettype($db)), E_USER_WARNING);
            return false;
        }
        if (!is_object($structure) || !is_a($structure, "DbStructure")) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 2, 'Instance of DbStructure', gettype($structure)), E_USER_WARNING);
            return false;
        }


    }
    
    function renameColumn()
    {
        '';
    }
}
// Copyright  2009  Thomas Meyer  http://www.yanaframework.net
?>