<?php
/**
 * YANA library
 *
 *
 * Software:        Yana PHP-Framework 3.0.0
 * Author:          Dipl. Inf. Thomas Meyer
 * Homepage:        http://www.yanaframework.net
 * Copyright:       (c) 2009  Thomas Meyer
 * License:         GNU GPL  http://www.gnu.org/licenses/
 *
 * This program: can be redistributed and/or modified under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/.
 *
 * You may use this software for free. In case you like it, or want to
 * use it for a commercial purpose, please consider making a donation.
 * http://yanaframework.net/donate.php
 *
 * This notice MAY NOT be removed.
 *
 *
 * @package  yana
 * @license  http://www.gnu.org/licenses/gpl.txt
 *
 * @ignore
 */

/**
 * FloodFile
 *
 * This file is intended to be used as a counter.
 *
 * @access      public
 * @package     yana
 * @subpackage  file_system
 *
 * @ignore
 */
class FloodFile extends File
{

    /**
     * @ignore
     * @access  private
     * @var     int */
    var $max = 0;

    /**
     * add entry to file
     *
     * increment counter and save changes
     *
     * @access  public
     * @return  bool
     */
    function set()
    {
        if (!$this->exists()) {
            $this->create();
        }
        global $YANA;
        if (isset($YANA)) {
            $REMOTE_ADDR = $YANA->getVar('REMOTE_ADDR');
        } elseif (isset($_SERVER['REMOTE_ADDR'])) {
            $REMOTE_ADDR = $_SERVER['REMOTE_ADDR'];
        } else {
            $REMOTE_ADDR = '0.0.0.0';
        }

        if (!$this->isEmpty() && $REMOTE_ADDR != preg_replace("/\s/", "", $this->content[0]) || ($this->content[1] +20000) < time()) {
            $this->content[0] = $REMOTE_ADDR ."\n";
            $this->content[1] = time() ."\n";
            $this->content[2] = 1;
        } else {
            $this->content[1] = time() ."\n";
            $this->content[2] += 1;
        }
        $test = $this->write();
        assert('is_bool($test);');
        return $test;
    }

    /**
     * alias of set()
     *
     * @access  public
     */
    function insert()
    {
        $this->set();
    }

    /**
     * check if user is blocked
     *
     * Returns bool(true) if the maximum number of entries
     * for this user have been exceeded and bool(false)
     * otherwise.
     *
     * @access  public
     * @return  bool
     */
    function isBlocked()
    {
        $this->read();
        switch (true)
        {
            /**
             * 1) is deactivated
             */
            case $this->max <= 0:
            /**
             * 2) no data found (yet) - occurs during first run
             */
            case $this->isEmpty():
            /**
             * 3) dataset is invalid (unexpected)
             */
            case $this->length() < 3:
            /**
             * 4) user is other than the last
             */
            case strpos($this->content[0], $_SERVER['REMOTE_ADDR']) === false:
            /**
             * 5) maximum number of entries not reached yet
             */
            case $this->content[2] < $this->max:
                return false;
            break;
            /**
             * 6) user has exceeded maximum number of entries
             */
            default:
                return true;
            break;
        } /* end switch */    
    }

    /**
     * set maxmimum number of entries
     *
     * Returns bool(true) on success and bool(false) on error.
     *
     * @access  public
     * @param   int  $max  a positive number
     * @return  bool
     */
    function setMax($max)
    {
        /**
         * Input checking
         */
        if (!is_int($max)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'Integer', gettype($max)), E_USER_WARNING);
            return false;
        }

        if ($max >= 0) {
            $this->max = $max;
            return true;
        } else {
            trigger_error("Expected maximum number to be a positive integer, found '${max}' instead.", E_USER_NOTICE);
            return false;
        }
    }

    /**
     * get maximum number of entries
     *
     * @access  public
     * @return  int
     */
    function getMax()
    {
        assert('is_int($this->max);');
        return $this->max;
    }

}
// Copyright  2009  Thomas Meyer  http://www.yanaframework.net
?>