<?php
/**
 * YANA library
 *
 *
 * Software:        Yana PHP-Framework 3.0.0
 * Author:          Dipl. Inf. Thomas Meyer
 * Homepage:        http://www.yanaframework.net
 * Copyright:       (c) 2009  Thomas Meyer
 * License:         GNU GPL  http://www.gnu.org/licenses/
 *
 * This program: can be redistributed and/or modified under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/.
 *
 * You may use this software for free. In case you like it, or want to
 * use it for a commercial purpose, please consider making a donation.
 * http://yanaframework.net/donate.php
 *
 * This notice MAY NOT be removed.
 *
 *
 * @package  yana
 * @license  http://www.gnu.org/licenses/gpl.txt
 */

/**
 * abstract Plugin
 *
 * This is an abstract super-class for all plugins.
 * Each subclass should re-implement the abstract
 * default event handler.
 *
 * @abstract
 * @access      public
 * @package     yana
 * @subpackage  core
 */
class Plugin
{

    /**
     * @ignore
     * @access  private
     * @var     string */ var $name = "";

    /**
     * constructor
     *
     * Creates a new instance.
     * When reimplementing this function,
     * note that you should at least set
     * the private name attribute, as this
     * serves as an id and therefore it
     * is essential to communicate with other
     * plugins.
     *
     * @param  string  $name  a humean readable, unique identifier for this instance
     */
    function Plugin($name)
    {
        assert('is_string($name); /* plugin name needs to be a string '.gettype($name).' */');
        assert('!empty($name);    /* plugin name cannot be empty */');
        /* settype to STRING */
        $name = (string) $name;
        $this->name = $name;
    }

    /**
     * abstract  default event handler
     *
     * Since this is an abstract function it should
     * be reimplemented in all subclasses
     *
     * @access   public
     * @abstract
     * @param    string  $event  identifier of the occured event
     * @param    array   $ARGS   list of arguments
     * @return   bool
     */
    function _default ($event, $ARGS)
    {
        /* intentionaly left blank */
        return true;
    }

    /**
     * get the name of a plugin
     *
     * The "name" of a plugin is also a unique identifier
     * for it. This means it is an alphanumeric, lower-cased string.
     * Plugin names can also be used to communicate with the
     * PluginManager singleton class to reference to a specific
     * plugin.
     *
     * @access  public
     * @return  string
     */
    function getName()
    {
        assert('is_string($this->name) && !empty($this->name);');
        return $this->name;
    }

}
// Copyright  2009  Thomas Meyer  http://www.yanaframework.net
?>