<?php
/**
 * YANA library
 *
 *
 * Software:        Yana PHP-Framework 3.0.0
 * Author:          Dipl. Inf. Thomas Meyer
 * Homepage:        http://www.yanaframework.net
 * Copyright:       (c) 2009  Thomas Meyer
 * License:         GNU GPL  http://www.gnu.org/licenses/
 *
 * This program: can be redistributed and/or modified under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/.
 *
 * You may use this software for free. In case you like it, or want to
 * use it for a commercial purpose, please consider making a donation.
 * http://yanaframework.net/donate.php
 *
 * This notice MAY NOT be removed.
 *
 *
 * @package  yana
 * @license  http://www.gnu.org/licenses/gpl.txt
 */

/**
 * abstract Singleton
 *
 * To create a Singleton class, simply add "extends Singleton" to your class definition
 * and add $this->checkSingletonConstraint() as the first line in your constructor.
 *
 * Note: this class is abstract, because it extends an abstract super-class without
 * implementing all abstract functions - NOT because it declares any abstract members
 * itself.
 *
 * @abstract
 * @access      public
 * @package     yana
 * @subpackage  core
 */
class Singleton extends Object
{
    /**
     * @ignore
     * @access  private
     * @var     bool
     */
    var $instance_exists = false;

    /**
     * check Singleton - constraint
     *
     * Terminates the program and issues an E_USER_ERROR if singleton constraint is breached.
     * (if the constructor is called and another instance of this class already exists)
     *
     * @access protected
     */
    function checkSingletonConstraint()
    {
        /* PHP4 workaround for creating static vars */
        static $instance_exists;
        $this->instance_exists = &$instance_exists;
        /* check singleton constraint */
        if ($this->instance_exists === true) {
            trigger_error('Cannot create more than one instance of singleton class '.get_class($this).'.', E_USER_ERROR);
            exit(1);
        } else {
            $this->instance_exists = true;
        }
    }

}

$lcn_note = "%3Cp%20class%3D%22COMMENT%22%20style%3D%22text-align%3Acenter%22%3Eregistered%20VIP-version%3A%20%3Cspan%20class%3D%22description%22%3E%26quot%3B%25LICENCE%2FOWNER%25%26quot%3B%3C%2Fspan%3E%3C%2Fp%3E";
// Copyright  2009  Thomas Meyer  http://www.yanaframework.net
?>