<?php
/**
 * YANA library
 *
 *
 * Software:        Yana PHP-Framework 3.0.0
 * Author:          Dipl. Inf. Thomas Meyer
 * Homepage:        http://www.yanaframework.net
 * Copyright:       (c) 2009  Thomas Meyer
 * License:         GNU GPL  http://www.gnu.org/licenses/
 *
 * This program: can be redistributed and/or modified under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/.
 *
 * You may use this software for free. In case you like it, or want to
 * use it for a commercial purpose, please consider making a donation.
 * http://yanaframework.net/donate.php
 *
 * This notice MAY NOT be removed.
 *
 *
 * @package  yana
 * @license  http://www.gnu.org/licenses/gpl.txt
 *
 * @ignore
 */

/**
 * Singleton Skin
 *
 * @access      public
 * @package     yana
 * @subpackage  core
 *
 * @ignore
 */
class Skin extends Singleton
{

    /**#@+
     * @ignore
     * @access  private
     */

    /** @var string */ var $directory = "";
    /** @var Dir    */ var $dirStream = null;
    /** @var SML[]  */ var $files = array();
    /** @var array  */ var $value = array();

    /**#@-*/

    /**
     * Singleton Constructor
     *
     * @param  string  $directory
     * @param  Dir     $dirStream
     * @param  Dir     $skinDir
     * @param  Dir     $defaultDir
     *
     * @ignore
     */
    function Skin($directory, $dirStream, $skinDir, $defaultDir)
    {
        $this->checkSingletonConstraint();
        assert('is_string($directory); // Wrong argument type argument 1. String expected in new ' . __CLASS__ . '().');
        assert('is_dir($directory); // There is no skin directory "' . $directory . '". Will use default skin instead.');

        /* settype to STRING */
        $directory = (string) $directory;

        $this->directory = $directory;
        $this->dirStream = $dirStream;
        $this->dirStream->read();

        /* Load Defaults first */
        if ($defaultDir->getPath() != $skinDir->getPath()){
            $content = $defaultDir->get();
            assert('!isset($filename); /* cannot redeclare variable $filename */');
            foreach ($content as $filename)
            {
                $this->files[$filename] = new SML($defaultDir->getPath().$filename, CASE_UPPER);
                $this->files[$filename]->read();
                $REF = $this->files[$filename]->get("*");
                assert('!isset($key); /* cannot redeclare variable $key */');
                assert('!isset($element); /* cannot redeclare variable $element */');
                foreach ($REF as $key => $element)
                {
                    if (!empty($element['FILE'])) {
                        $this->value[$key]['FILE'] = "../../".$defaultDir->getPath().$element['FILE'];
                    }
                    if (!empty($element['STYLE'])) {
                        skin::_fallback($this->value[$key]['STYLE'], $element['STYLE'], $defaultDir->getPath());
                    }
                    if (!empty($element['SCRIPT'])) {
                        skin::_fallback($this->value[$key]['SCRIPT'], $element['SCRIPT'], $defaultDir->getPath());
                    }
                    if (!empty($element['LANGUAGE'])) {
                        skin::_fallback($this->value[$key]['LANGUAGE'], $element['LANGUAGE'], false);
                    }
                } /* end foreach */
                unset($key, $element);
            } /* end foreach */
            unset($filename);
        }

        /* Overwrite Defaults where needed */
        $content = $skinDir->get();
        assert('!isset($filename); /* cannot redeclare variable $filename */');
        foreach ($content as $filename)
        {
            $this->files[$filename] = new SML($skinDir->getPath().$filename, CASE_UPPER);
            $this->files[$filename]->read();
            $REF = $this->files[$filename]->get("*");
            assert('!isset($key); /* cannot redeclare variable $key */');
            assert('!isset($element); /* cannot redeclare variable $element */');
            foreach ($REF as $key => $element)
            {
                if (!empty($element['FILE'])) {
                    $this->value[$key]['FILE'] = $element['FILE'];
                }
                if (!empty($element['STYLE'])) {
                    skin::_fallback($this->value[$key]['STYLE'], $element['STYLE'], $skinDir->getPath());
                }
                if (!empty($element['SCRIPT'])) {
                    skin::_fallback($this->value[$key]['SCRIPT'], $element['SCRIPT'], $skinDir->getPath());
                }
                if (!empty($element['LANGUAGE'])) {
                    skin::_fallback($this->value[$key]['LANGUAGE'], $element['LANGUAGE'], false);
                }
            }  /* end foreach */
            unset($key, $element);
        } /* end foreach */
        unset($filename);
    }

    /**
     * get
     *
     * @access  public
     * @param   string  $reference  (optional)
     * @param   string  $key        (optional)
     * @return  array
     */
    function get($reference = "*", $key = "*")
    {
        /* settype to STRING */
        $reference = (string) $reference;
        $key       = (string) $key;

        $REF = array();
        if (empty($key)) {
            $key = "*";
        }
        if (empty($reference)) {
            $reference = "*";
        }
        $key       = strtoupper($key);
        $reference = strtoupper($reference);

        if ($reference == "*") {
            assert('!isset($file); /* cannot redeclare variable $file */');
            foreach ($this->files as $file)
            {
                $REF = Hashtable::merge($REF, $file->get($key));
            }
            unset($file);
            return $REF;
        } elseif (!isset($this->files[$reference])) {
            return false;
        } else {
            return $this->files[$reference]->get($key);
        }

    }

    /**
     * get filename
     *
     * This returns the path and name of the template file associated with $key,
     * or bool(false) if there is none.
     *
     * @access  public
     * @param   string  $key
     * @return  string|bool
     */
    function getFile($key)
    {
        /* input check */
        if (!is_string($key)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($key)), E_USER_WARNING);
            return false;
        } elseif (empty($key)) {
            /* nothing to do */
            return false;
        } else {
            $key = strtoupper($key);
        }

        if (!isset($this->value[$key]['FILE'])) {
            return false;
        } else {
            $filename = $this->value[$key]['FILE'];
        }

        global $YANA;

        if (isset($YANA)) {

            /* load language files associated with the template */
            $languageList = $this->getLanguage($key);
            assert('!isset($language_file); // Cannot redclare $language_file');
            foreach ($languageList as $language_file)
            {
                $YANA->language->read($language_file);
            }
            unset($language_file);

            /* prepare a list of css styles associated with the template */
            $styleList = $this->getStyle($key);
            if (is_array($styleList)) {
                $oldStyleList = $YANA->view->get('SYSTEM_STYLE');
                if (is_array($oldStyleList)) {
                    $styleList = array_merge($oldStyleList, $styleList);
                }
                $styleList = array_unique($styleList);
                $YANA->view->insert('SYSTEM_STYLE', $styleList);
            }

            /* prepare a list of javascript files associated with the template */
            $scriptList = $this->getScript($key);
            if (is_array($scriptList)) {
                $oldScriptList = $YANA->view->get('SYSTEM_SCRIPT');
                if (is_array($oldScriptList)) {
                    $scriptList = array_merge($oldScriptList, $scriptList);
                }
                $YANA->view->insert('SYSTEM_SCRIPT', $scriptList);
            }

        } /* end if */

        /* return the name of the file */
        return $this->getDirectory() . $filename;

    }

    /**
     * check if input is a valid template id
     *
     * Returns bool(true) if a template named $key is currently registered
     * and bool(false) otherwise.
     *
     * @access  public
     * @param   string  $key
     * @return  bool
     */
    function isId($key)
    {
        /* input check */
        if (!is_string($key)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($key)), E_USER_WARNING);
            return false;
        } elseif (empty($key)) {
            /* nothing to do */
            return false;
        } else {
            $key = strtoupper($key);
        }

        return isset($this->value[$key]['FILE']);
    }

    /**
     * get language
     *
     * @access  public
     * @param   string  $key
     * @return  array
     */
    function &getLanguage($key)
    {
        /* input check */
        if (!is_string($key)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($key)), E_USER_WARNING);
            $result = array();
            return $result;
        } elseif (empty($key)) {
            /* nothing to do */
            $result = array();
            return $result;
        } else {
            $key = strtoupper($key);
        }

        if (!isset($this->value[$key])) {
            $result = array();
            return $result;
        }

        $file_definition =& $this->value[$key];
        $languageList   =& $this->value[$key]['LANGUAGE'];

        if (!is_null($languageList)) {
            if (is_array($languageList)) {
                return $languageList;
            } elseif (is_string($languageList)) {
                $result = array(&$languageList);
                return $result;
            }
        } else {
            $result = array();
            return $result;
        }
    }

    /**
     * set stylesheet
     *
     * This sets a CSS-stylesheet for template $key.
     *
     * Examples:
     * <code>
     * // add stylesheet
     * $skin->setStyle('foo', 'foo.css');
     * // replace stylesheet 'bar'
     * $skin->setStyle('foo', 'bar.css', 'bar');
     * // remove stylesheet 'bar'
     * $skin->setStyle('foo', null, 'bar');
     * // remove all stylesheets
     * $skin->setStyle('foo');
     * </code>
     *
     * Returns bool(true) on success and bool(false) on error.
     *
     * @access  public
     * @param   string  $key
     * @param   string  $file
     * @param   string  $name
     * @return  bool
     * @since   2.9.8
     */
    function setStyle($key, $file = null, $name = null)
    {
        /* input check */
        if (!is_string($key)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($key)), E_USER_WARNING);
            return false;
        } elseif (empty($key)) {
            /* nothing to do */
            return false;
        } else {
            $key = strtoupper($key);
        }
        if (!isset($this->value[$key])) {
            return false;
        }
        if (is_null($file)) {
            /* intentionally left blank */
        } elseif (!is_string($file)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 2, 'String', gettype($file)), E_USER_WARNING);
            return false;
        }
        if (!is_string($name) && !is_null($name)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 3, 'String', gettype($name)), E_USER_WARNING);
            return false;
        }

        return $this->setProperty($key, 'STYLE', $file, $name);
    }

    /**
     * set javascript
     *
     * This sets a javascript-file for template $key.
     *
     * Examples:
     * <code>
     * // add script
     * $skin->setScript('foo', 'foo.js');
     * // replace script 'bar'
     * $skin->setScript('foo', 'bar.js', 'bar');
     * // remove script 'bar'
     * $skin->setScript('foo', null, 'bar');
     * // remove all scripts
     * $skin->setScript('foo');
     * </code>
     *
     * Returns bool(true) on success and bool(false) on error.
     *
     * @access  public
     * @param   string  $key
     * @param   string  $file
     * @param   string  $name
     * @return  bool
     * @since   2.9.8
     */
    function setScript($key, $file = null, $name = null)
    {
        /* input check */
        if (!is_string($key)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($key)), E_USER_WARNING);
            return false;
        } elseif (empty($key)) {
            /* nothing to do */
            return false;
        } else {
            $key = strtoupper($key);
        }
        if (!isset($this->value[$key])) {
            return false;
        }
        if (is_null($file)) {
            /* intentionally left blank */
        } elseif (!is_string($file)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 2, 'String', gettype($file)), E_USER_WARNING);
            return false;
        }
        if (!is_string($name) && !is_null($name)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 3, 'String', gettype($name)), E_USER_WARNING);
            return false;
        }

        return $this->setProperty($key, 'SCRIPT', $file, $name);
    }

    /**
     * set language
     *
     * This sets a language-file for template $key.
     *
     * Examples:
     * <code>
     * // add language
     * $skin->setLanguage('foo', 'foo.config');
     * // replace language 'bar'
     * $skin->setLanguage('foo', 'bar.config', 'bar');
     * // remove language 'bar'
     * $skin->setLanguage('foo', null, 'bar');
     * // remove all language-files
     * $skin->setLanguage('foo');
     * </code>
     *
     * Returns bool(true) on success and bool(false) on error.
     *
     * @access  public
     * @param   string  $key
     * @param   string  $file
     * @param   string  $name
     * @return  bool
     * @since   2.9.8
     */
    function setLanguage($key, $file = null, $name = null)
    {
        /* input check */
        if (!is_string($key)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($key)), E_USER_WARNING);
            return false;
        } elseif (empty($key)) {
            /* nothing to do */
            return false;
        } else {
            $key = strtoupper($key);
        }
        if (!isset($this->value[$key])) {
            return false;
        }
        if (is_null($file)) {
            /* intentionally left blank */
        } elseif (!is_string($file)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 2, 'String', gettype($file)), E_USER_WARNING);
            return false;
        }
        if (!is_string($name) && !is_null($name)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 3, 'String', gettype($name)), E_USER_WARNING);
            return false;
        }

        return $this->setProperty($key, 'LANGUAGE', $file, $name);
    }

    /**
     * set property
     *
     * This sets a property for template $key.
     *
     * Returns bool(true) on success and bool(false) on error.
     *
     * @access  private
     * @param   string  $key
     * @param   string  $propertyName
     * @param   string  $file
     * @param   string  $name
     * @return  bool
     * @since   2.9.8
     */
    function setProperty($key, $propertyName, $file = null, $name = null)
    {
        /* input check */
        assert('is_string($key); // Wrong argument type argument 1. String expected');
        assert('is_string($propertyName); // Wrong argument type argument 2. String expected');
        assert('is_string($file) || is_null($file); // Wrong argument type argument 3. String expected');
        assert('is_string($name) || is_null($name); // Wrong argument type argument 4. String expected');

        $key = strtoupper($key);
        if (!isset($this->value[$key])) {
            return false;
        }
        $propertyName = strtoupper($propertyName);

        /**
         * 1) modify whole list
         */
        if (is_null($name)) {

            /**
             * 1.1) unset list
             */
            if (is_null($file)) {
                if (isset($this->value[$key][$propertyName])) {
                    unset($this->value[$key][$propertyName]);
                }
                return true;

            /**
             * 1.2) add file
             */
            } else {
                if (!isset($this->value[$key][$propertyName])) {
                    $this->value[$key][$propertyName] = array();
                }
                $this->value[$key][$propertyName][] = $file;
                return true;

            }

        /**
         * 2) modify particular entry
         */
        } elseif (!isset($this->value[$key][$propertyName]) || !is_array($this->value[$key][$propertyName])) {
            return false;

        } else {
            /**
             * 2.1) unset file
             */
            if (is_null($file)) {
                if (isset($this->value[$key][$propertyName][$name])) {
                    unset($this->value[$key][$propertyName][$name]);
                    return true;

                } else {
                    return false;
                }

            /**
             * 2.2) replace file
             */
            } else {
                if (!isset($this->value[$key][$propertyName])) {
                    $this->value[$key][$propertyName] = array();
                }
                $this->value[$key][$propertyName][$name] = $file;
                return true;

            }
        }
    }

    /**
     * get style
     *
     * Returns the definition of the identified stylesheet.
     *
     * @access  public
     * @param   string  $key
     * @return  array
     */
    function getStyle($key)
    {
        /* input check */
        if (!is_string($key)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($key)), E_USER_WARNING);
            return array();
        } elseif (empty($key)) {
            /* nothing to do */
            return array();
        } else {
            $key = strtoupper($key);
        }

        if (!isset($this->value[$key])) {
            return array();
        }

        $file_definition =& $this->value[$key];
        $styleList      = @$file_definition['STYLE'];

        if (!is_array($styleList)) {
            return array();
        } else {
            return $styleList;
        }
    }

    /**
     * get script
     *
     * @access  public
     * @param   string  $key
     * @return  array
     */
    function getScript($key)
    {
        /* input check */
        if (!is_string($key)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($key)), E_USER_WARNING);
            return false;
        } elseif (empty($key)) {
            /* nothing to do */
            return false;
        } else {
            $key = strtoupper($key);
        }

        $file_definition =& $this->value[$key];
        if (isset($file_definition['SCRIPT'])) {
            $scriptList = $file_definition['SCRIPT'];
        } else {
            $scriptList = "";
        }

        if (empty($key)) {
            return false;
        } elseif (!is_array($scriptList)) {
            return array();
        } else {
            return $scriptList;
        }
    }

    /**
     * get content
     *
     * @access  public
     * @return  array  (numeric array of strings)
     */
    function getContent()
    {
        $content = $this->dirStream->getContent();
        assert('is_array($content);');
        if (is_array($content)) {
            return $content;
        } else {
            return array();
        }
    }

    /**
     * get directory
     *
     * @access  public
     * @return  string
     */
    function getDirectory()
    {
        assert('is_string($this->directory);');
        return $this->directory;
    }

    /**
     * get info
     *
     * @access  public
     * @param   string  $filename
     * @return  array
     */
    function getInfo($filename)
    {
        /* input check */
        if (!is_string($filename)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($filename)), E_USER_WARNING);
            return false;
        }

        $file = ($this->dirStream->getPath()).$filename;
        if (is_file($file)) {
            $REF = SML::getFile($file, CASE_UPPER);
            assert('isset($REF["SKIN_INFO"]) && is_array($REF["SKIN_INFO"]); /* the resource is not a valid skin info file */');
            if (is_array($REF['SKIN_INFO'])) {
                $REF['SKIN_INFO']['LAST_CHANGE'] = filemtime($file);
                clearstatcache();
                return $REF["SKIN_INFO"];
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

    /**
     *  _fallback
     *
     * implements fallback behaviour
     *
     * @access  private
     * @param   array|null    &$list
     * @param   array|string  &$new
     * @param   string|bool   $prefix
     * @return  array
     * @ignore
     */
    function _fallback(&$list, &$new, $prefix = "")
    {
        if (!empty($new)) {

            /* settype to ARRAY */
            $new  = (array) $new;
            $list = (array) $list;

            /* overwrite defaults where required */
            foreach (array_keys($new) as $i)
            {
                $var =& $new[$i];
                if (is_string($prefix)) {
                    $var = $prefix.$new[$i];
                }
                /* NOTE: Collisions are treated as follows.
                 * Numeric indices will always be appended, no matter what, while textual indices will get replaced.
                 *
                 * This allows the user to decide wether to create an 'anonymous' association (numbered index)
                 * or a named association (textual index) with a stylesheet.
                 *
                 * So in a derived template the user may decide for himself what element to take and what to drop.
                 *
                 * Also note, that you can drop a named association in a derived template by entering string('null') or boolean('false') as source.
                 */
                if (is_int($i) && ($prefix === false || is_file($var))) {
                    array_push($list, $var);
                } elseif ($var == 'null' || empty($var)) {
                    $list[$i] = false;
                } elseif ($prefix === false || is_file($var)) {
                    $list[$i] = $var;
                } else {
                    $list[$i] = null;
                    trigger_error("The provided value '${i}' => '${var}' is not a valid file resource.", E_USER_WARNING);
                }
            }
        } else {
            /* intentionally left blank */
        }
    }

    /**
     * create and return a report on this instance
     *
     * Returns the report as a string. The argument $asHtml allows
     * you to decide, wether the result should be HTML or plain text.
     *
     * HTML will use the following CSS classes:
     * <ul>
     * <li> div.error - encloses an error message </li>
     * <li> div.error span.label - some heading (e.g. "Error") </li>
     * <li> div.error span.description - some text that describes the error </li>
     * </ul>
     *
     * The text will be presented in English, using charset Latin-1.
     *
     * If the resulting string is empty, no errors occured.
     *
     * Example of usage:
     * <code>
     * $errors = $instance->getReport();
     *
     * if (empty($errors)) {
     * print 'all fine';
     * } else {
     * print 'The following errors were reported:'.$errors;
     * }
     * </code>
     *
     * @access  public
     * @param   bool  $asHtml
     * @return  string
     *
     * @name    Skin::getReport()
     * @ignore
     */
    function getReport($asHtml = true)
    {
        /* String */ $txt = "";
        global $YANA;

        foreach ($this->value as $key => $element)
        {
            $filename = $this->getFile($key);
            if (!empty($filename) && !file_exists($filename)) {
                $txt .= '<div class="error"><span class="label">Error:</span>'.
                '<span class="description">Skin entry \'' . $key . '\' references to file \'' . $filename . '\'. But this file does not exist.' .
                'Please make sure this path and filename is correct and you have all files installed. Reinstall this skin if necessary.</span>' .
                "</div>\n";
            }
            /* check language references */
            if (isset($YANA)) {
                if (!empty($element['LANGUAGE']) && is_array($element['LANGUAGE']) && count($element['LANGUAGE']) > 0) {
                    assert('!isset($value); /* cannot redeclare variable $value */');
                    foreach ($element['LANGUAGE'] as $value)
                    {
                        if (!empty($value)) {
                            if (!$YANA->language->read($value)) {
                                $txt .= '<div class="error"><span class="label">Error:</span>'.
                                "<span class=\"description\">A required language file '${value}.config' for template '${key}' is not available." .
                                'This template may not be displayed correctly. ' .
                                'Please check if the chosen language file is correct and update your language pack if needed.</span>' .
                                "</div>\n";
                            }
                        }
                    }
                    unset($value);
                }
            } /* end if */
            /* check stylesheet references */
            if (!empty($element['STYLE']) && is_array($element['STYLE']) && count($element['STYLE']) > 0) {
                assert('!isset($value); /* cannot redeclare variable $value */');
                foreach ($element['STYLE'] as $value)
                {
                    if (is_null($value)) {
                        $txt .= '<div class="error"><span class="label">Error:</span>'.
                        "<span class=\"description\">A required stylesheet for template '${key}' is not available." .
                        'This template may not be displayed correctly.</span>' .
                        "</div>\n";
                    }
                }
                unset($value);
            }
            /* check script references */
            if (!empty($element['SCRIPT']) && is_array($element['SCRIPT']) && count($element['SCRIPT']) > 0) {
                assert('!isset($value); /* cannot redeclare variable $value */');
                foreach ($element['SCRIPT'] as $value)
                {
                    if (is_null($value)) {
                        $txt .= '<div class="error"><span class="label">Error:</span>'.
                        "<span class=\"description\">A required javascript file for template '${key}' is not available." .
                        'This template may not be displayed correctly.</span>' .
                        "</div>\n";
                    }
                }
                unset($value);
            }
        } /* end foreach */

        if ($asHtml) {
            return $txt;
        } else {
            return strip_tags($txt);
        }
    }

    /**
     * self test
     *
     * This function exists only for debugging purposes.
     * Do not uncomment in productive environments.
     *
     * @access  public
     * @static
     * @param   int  $nr  number of test to take
     * @return  bool
     * @ignore
     */
    function test($nr = 1)
    {
        global $YANA;
        switch ($nr)
        {
            case 1:
                $skin =& $YANA->skin;
                $dir = $YANA->getVar('SKINDIR');

                // test Skin::setStyle

                // reset template foo
                $skin->value['FOO'] = array();
                $temp = array();

                // add stylesheet
                $test = $skin->setStyle('foo', 'foo.css');
                $temp[] = 'foo.css';
                if (!$test || ($skin->getStyle('foo') !== $temp)) {
                    return "add stylesheet failed";
                }

                // replace stylesheet 'bar'
                $test = $skin->setStyle('foo', 'bar.css', 'bar');
                $temp['bar'] = 'bar.css';
                if (!$test || ($skin->getStyle('foo') !== $temp)) {
                    return "replace stylesheet 'bar' failed";
                }

                // remove stylesheet 'bar'
                $test = $skin->setStyle('foo', null, 'bar');
                unset($temp['bar']);
                if (!$test || ($skin->getStyle('foo') !== $temp)) {
                    return "remove stylesheet 'bar' failed";
                }

                // remove all stylesheets
                $test = $skin->setStyle('foo');
                $temp = array();
                if (!$test || ($skin->getStyle('foo') !== $temp)) {
                    return "remove all stylesheets";
                }

                // test Skin::setScript

                // reset template foo
                $skin->value['FOO'] = array();
                $temp = array();

                // add script
                $test = $skin->setScript('foo', 'foo.js');
                $temp[] = 'foo.js';
                if (!$test || ($skin->getScript('foo') !== $temp)) {
                    return "add script failed";
                }

                // replace script 'bar'
                $test = $skin->setScript('foo', 'bar.js', 'bar');
                $temp['bar'] = 'bar.js';
                if (!$test || ($skin->getScript('foo') !== $temp)) {
                    return "replace script 'bar' failed";
                }

                // remove script 'bar'
                $test = $skin->setScript('foo', null, 'bar');
                unset($temp['bar']);
                if (!$test || ($skin->getScript('foo') !== $temp)) {
                    return "remove script 'bar' failed";
                }

                // remove all scripts
                $test = $skin->setScript('foo');
                $temp = array();
                if (!$test || ($skin->getScript('foo') !== $temp)) {
                    return "remove all scripts";
                }

                // test Skin::setLanguage

                // reset template foo
                $skin->value['FOO'] = array();
                $temp = array();

                // add language
                $test = $skin->setLanguage('foo', 'foo.config');
                $temp[] = 'foo.config';
                if (!$test || ($skin->getLanguage('foo') !== $temp)) {
                    return "add language";
                }

                // replace language 'bar'
                $test = $skin->setLanguage('foo', 'bar.config', 'bar');
                $temp['bar'] = 'bar.config';
                if (!$test || ($skin->getLanguage('foo') !== $temp)) {
                    return "replace language 'bar' failed";
                }

                // remove language 'bar'
                $test = $skin->setLanguage('foo', null, 'bar');
                unset($temp['bar']);
                if (!$test || ($skin->getLanguage('foo') !== $temp)) {
                    return "remove language 'bar' failed";
                }

                // remove all language-files
                $test = $skin->setLanguage('foo');
                $temp = array();
                if (!$test || ($skin->getLanguage('foo') !== $temp)) {
                    return "remove all language-files";
                }

                return true;
            break;
        }
    } /* end of function */
}
// Copyright  2009  Thomas Meyer  http://www.yanaframework.net
?>