<?php
/**
 * YANA library
 *
 *
 * Software:        Yana PHP-Framework 3.0.0
 * Author:          Dipl. Inf. Thomas Meyer
 * Homepage:        http://www.yanaframework.net
 * Copyright:       (c) 2009  Thomas Meyer
 * License:         GNU GPL  http://www.gnu.org/licenses/
 *
 * This program: can be redistributed and/or modified under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/.
 *
 * You may use this software for free. In case you like it, or want to
 * use it for a commercial purpose, please consider making a donation.
 * http://yanaframework.net/donate.php
 *
 * This notice MAY NOT be removed.
 *
 *
 * @package    yana
 * @subpackage vdrive
 * @license    http://www.gnu.org/licenses/gpl.txt
 */

/**
 * VDrive_dir
 *
 * class representing virtual directories
 *
 * @access     public
 * @package    yana
 * @subpackage vdrive
 *
 * @ignore
 */
class VDrive_dir extends VDrive_mountpoint {

    /**
     * Note: this overwrites the variable of the parent class
     *
     * @access  private
     * @var     string
     * @ignore
     */
    var $type = "dir";

    /**
     * constructor
     *
     * @param  array   $SRC
     * @param  string  $type
     * @param  bool    $automount
     * @param  string  $filter
     */
    function VDrive_dir($SRC, $type, $automount, $filter = "")
    {
        /* settype to ARRAY
         *            STRING
         *            BOOLEAN
         *            STRING
         */
        $SRC       = (array)  $SRC;
        $type      = (string) $type;
        $automount = (bool)   $automount;
        $filter    = (string) $filter;

        switch ($type) {
            case "dir":
                $this->mountpoint = new Dir($SRC[0]);
                $this->mountpoint->filter = $filter;
                if($automount) {
                    $this->mount();
                }
            break;
            default:
                trigger_error("Invalid directory-type argument '$type' supplied for Mountpoint '" . print_r($SRC[0], true) . "'.", E_USER_WARNING);
            break;
        }

    }

    /**
     * mount the current resource
     *
     * Returns bool(true) on success and bool(false) on error.
     * In this case "mounting" means it calls the function "read()"
     * on the resource object, if it has it.
     *
     * Note: this overwrites the function of the parent class
     *
     * @access public
     * @return boolean
     */
    function mount()
    {
        assert('is_bool($this->is_mounted);');
        if ($this->isMounted()) {
            return true;
        } else {
            if (is_object($this->mountpoint) && method_exists($this->mountpoint, "read")) {
                $this->mountpoint->read();
                $this->is_mounted = true;
                return true;
            } else {
                return false;
            }
        }
    }

}
// Copyright  2009  Thomas Meyer  http://www.yanaframework.net
?>