<?php
/**
 * YANA library
 *
 *
 * Software:        Yana PHP-Framework 3.0.0
 * Author:          Dipl. Inf. Thomas Meyer
 * Homepage:        http://www.yanaframework.net
 * Copyright:       (c) 2009  Thomas Meyer
 * License:         GNU GPL  http://www.gnu.org/licenses/
 *
 * This program: can be redistributed and/or modified under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/.
 *
 * You may use this software for free. In case you like it, or want to
 * use it for a commercial purpose, please consider making a donation.
 * http://yanaframework.net/donate.php
 *
 * This notice MAY NOT be removed.
 *
 *
 * @package    yana
 * @subpackage vdrive
 * @license    http://www.gnu.org/licenses/gpl.txt
 */

/**
 * VDrive_file
 *
 * class representing virtual files
 *
 * @access     public
 * @package    yana
 * @subpackage vdrive
 *
 * @ignore
 */
class VDrive_file extends VDrive_mountpoint
{
    /**#@+
     * @ignore
     * @access  protected
     */

    /** @var array  */ var $src = array();
    /** @var bool   */ var $automount = false;
    /** @var int    */ var $caseSensitive = CASE_MIXED;

    /**#@-*/

    /**
     * constructor
     *
     * @param  array   $SRC
     * @param  string  $type
     * @param  bool    $automount
     */
    function VDrive_file ($SRC, $type, $automount)
    {
        /* settype to ARRAY
         *            STRING
         *            BOOLEAN
         */
        $this->src = (array)  $SRC;
        $type      = (string) $type;
        $automount = (bool)   $automount;

        if ($type === "file" || empty($type)) {
            $this->type = "FileReadonly";

        } elseif (preg_match('/^sml ?(case_upper|case_lower|case_mixed)?$/', strtolower($type), $m)) {
            $this->type = "SML";
            switch ($m[1])
            {
                case 'case_upper':
                    $this->caseSensitive = CASE_UPPER;
                break;
                case 'case_lower':
                    $this->caseSensitive = CASE_LOWER;
                break;
                default:
                    $this->caseSensitive = CASE_MIXED;
                break;
            } /* end switch */

        } elseif (!class_exists($type)) {
            trigger_error("Invalid resource-type argument supplied for Mountpoint '" . print_r($SRC, true) . "'.\n\t" .
            "No such file wrapper: '$type'.", E_USER_WARNING);
            return;

        } else {
            $this->type = $type;
            $case_sensitive = null;
        }

        if ($automount) {
            if ($this->load()) {
                $this->mount();
            }
        }

    }

    /**
     * load class wrapper
     *
     * This is abstract and should be overwritten in each subclass.
     *
     * @access  private
     * @return  bool
     */
    function load()
    {
        for ($i = 0; $i < count($this->src); $i++)
        {
            if (file_exists($this->src[$i])) {
                if ($this->type === 'SML') {
                    $this->mountpoint = new SML($this->src[$i], $this->caseSensitive);
                } else {
                    $this->mountpoint = new $this->type($this->src[$i]);
                }
                break;
            }
        } /* end for */

        if (!isset($this->mountpoint)) {
            $this->mountpoint = new $this->type(end($this->src));
            return true;

        } elseif (!is_subclass_of($this->mountpoint, "FileSystemResource")) {
            trigger_error("Invalid resource-type argument supplied for Mountpoint '" . print_r($this->src, true) . "'.\n\t" .
            "The wrapper-class '$type' cannot be used as a mountpoint.", E_USER_WARNING);
            unset($this->mountpoint);
            return false;

        } else {
            return true;

        }
    }

    /**
     * get mounted resource
     *
     * Returns the mounted file resource or bool(false) if none is present.
     *
     * @access public
     * @return object|bool(false)
     */
    function &getMountpoint()
    {
        $error = false;
        if (!$this->isLoaded() && !$this->load()) {
            return $error;
        } elseif (isset($this->mountpoint) && is_object($this->mountpoint)) {
            return $this->mountpoint;
        } else {
            return $error;
        }
    }

}
// Copyright  2009  Thomas Meyer  http://www.yanaframework.net
?>