<?php
/**
 * Plugin for Yana PHP-Framework
 *
 * Software:        Yana PHP-Framework 3.0.0
 * Author:          Dipl. Inf. Thomas Meyer
 * Homepage:        http://www.yanaframework.net
 * Copyright:       (c) 2009  Thomas Meyer
 * License:         GNU GPL  http://www.gnu.org/licenses/
 *
 * This program: can be redistributed and/or modified under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/.
 *
 * You may use this software for free. In case you like it, or want to
 * use it for a commercial purpose, please consider making a donation.
 * http://yanaframework.net/donate.php
 *
 * This notice MAY NOT be removed.
 *
 *
 * @author     Thomas Meyer
 * @license    http://www.gnu.org/licenses/gpl.txt
 * @package    yana
 * @subpackage plugins
 */

/**
 * load PEAR-DB
 * @ignore
 */
@include_once("DB.php");

/**
 * database administration tool
 *
 * @access     public
 * @package    yana
 * @subpackage plugins
 */
class plugin_db_admin extends Plugin
{

    /**
     * @access  private
     * @var     File
     * @ignore
     */
    var $file;

    /**
     * Constructor
     *
     * @access  public
     * @param   string  $name a name for this plugin
     * @ignore
     */
    function plugin_db_admin($name)
    {
        settype($name,"string");
        $this->name = $name;
        global $YANA;
        $this->file = $YANA->plugins->get($this->name,"DBCONFIGFILE");
    }

    /**
     * provide database setup form
     *
     * parameters taken: none
     *
     * @access  public
     * @param   array  $ARGS  array of params passed to the function
     * @return  bool
     */
    function get_db_configuration($ARGS)
    {
        unset($ARGS);

        global $YANA;

        if (!class_exists("DB")) {
            $YANA->report( new Error('PEARDB_FAILURE') );
            return false;
        }

        $YANA->setVar("DATABASE_ACTIVE",     YANA_DATABASE_ACTIVE);
        $YANA->setVar("DATABASE_DBMS",       YANA_DATABASE_DBMS);
        $YANA->setVar("DATABASE_HOST",       YANA_DATABASE_HOST);
        $YANA->setVar("DATABASE_PORT",       YANA_DATABASE_PORT);
        $YANA->setVar("DATABASE_USER",       YANA_DATABASE_USER);
        $YANA->setVar("DATABASE_PASSWORD",   YANA_DATABASE_PASSWORD);
        $YANA->setVar("DATABASE_PREFIX",     YANA_DATABASE_PREFIX);
        $YANA->setVar("DATABASE_NAME",       YANA_DATABASE_NAME);
        $YANA->setVar("YANA_DATABASE_ACTIVE",YANA_DATABASE_ACTIVE);
        $DATABASE_LIST = dirlist($YANA->getVar("DBDIR"), '*.config');
        foreach ($DATABASE_LIST as $key => $item)
        {
            $DATABASE_LIST[$key] = preg_replace("/\.\w*$/", "", $item);
        }
        $YANA->setVar("DATABASE_LIST", $DATABASE_LIST);
        return true;
    }

    /**
     * install databases
     *
     * parameters taken:
     *
     * <ul>
     * <li> string database_dbms the DBMS to install databases on </li>
     * <li> array database_list  a list of databases to install </li>
     * </ul>
     *
     * @access  public
     * @param   array  $ARGS array of params passed to the function
     * @return  bool
     */
    function db_install($ARGS)
    {
        settype($ARGS, "array");
        global $YANA;

        if (isset($ARGS['silent']) && $ARGS['silent'] === true) {
            $isSilent = true;
        } else {
            $isSilent = false;
        }

        if (!class_exists("DB")) {
            $isSilent || $YANA->report( new Error('PEARDB_FAILURE') );
            return false;
        }

        if (!isset($ARGS['database_dbms']) || !is_string($ARGS['database_dbms']) || !isset($ARGS['database_list']) || !is_array($ARGS['database_list']) || count($ARGS['database_list']) === 0) {
            /* invalid option - the choosen dbms is unknown */
            $isSilent || $YANA->report( new Alert('INPUT_INVALID'));
            return false;
        } else {
            $dbms = strtoupper($ARGS['database_dbms']);
            $dbList = $ARGS['database_list'];
        }

        /* Mapping the DBMS to the SQL export function in class DbCreator */
        switch ($dbms)
        {
            case 'DBASE':
            case 'FBSQL':
            case 'IBASE':
            case 'IFX':
            case 'SYBASE':
                $method_name  = null;
            break;
            case 'DB2':
                $method_name  = 'createDB2';
            break;
            case 'ACCESS':
                $method_name  = 'createMSAccess';
            break;
            case 'MSSQL':
                $method_name  = 'createMSSQL';
            break;
            case 'MYSQL':
            case 'MYSQLI':
                $dbms         = 'MYSQL';
                $method_name  = 'createMySQL';
            break;
            case 'OCI8':
                $method_name  = 'createOracleDB';
            break;
            case 'PGSQL':
                $method_name  = 'createPostgreSQL';
            break;
            default:
                $YANA->report( new Alert('INPUT_INVALID'));
                return false;
            break;
        }

        /* Mapping the DBMS to it's installation directory */
        $installDirectory = $YANA->registry->get($dbms);
        if ($installDirectory === false) {
            /* invalid option - the choosen dbms is unknown */
            $YANA->report( new Log("Unable to install database. The choosen DBMS '${dbms}' is unknown."));
            $isSilent || $YANA->report( new Alert('INPUT_INVALID'));
            return false;
        }

        /* we assume the class DbCreator has the desired method. This will be tested later! */

        if (!$installDirectory->exists() && is_null($method_name)) {
            $isSilent || $YANA->report( new Error("There is no installation file for this dbms available.") );
            return false;
        }

        /* get the list of available installation files */
        $installDirectory = $installDirectory->getPath();
        
        assert('!isset($initialization); // Cannot redeclare var $initialization');
        $initialization = array();

        foreach ($dbList as $item)
        {
            /* check the input */
            if (!is_string($item)) {
                $isSilent || $YANA->report( new Alert('INPUT_INVALID'));
                return false;
            } else {
                $item = strtolower($item);
            }

            $installFile   = $installDirectory . $item . '.sql';
            $structureFile = new DbStructure($item);
            $database      = new DbStream($structureFile);

            /* If no SQL file for the current $item does exist,
             * we need to call the appropriate DbCreator method
             * instead.
             */
            if (!is_readable($installFile)) {
                $dbCreator = new DbCreator($structureFile);

                /* If the DbCreator class does not support the desired function.
                 */
                if (!method_exists($dbCreator, $method_name)) {

                    if (!file_exists($installFile)) {
                        $isSilent || $YANA->report( new Error("Unable to install database. There is no installation file available for database '$item'.") );
                    } else {
                        $isSilent || $YANA->report( new Error("Unable to install database '$item'. Cannot read sql file '$installFile'.") );
                    }
                    return false;

                /* Else, create and execute the SQL statements.
                 */
                } else {
                    /* create ... */
                    $sqlStmts = $dbCreator->$method_name();
                    /* ... execute */
                    if (!$database->importSQL($sqlStmts)) {
                        /* if execution fails */
                        if (strpos($database->getErrorMessage(), 'already exists') === false) {
                            $isSilent || $YANA->report(new Error("SQL installation fault. Unable to install database '$item'."));
                            return false;

                        /* already exists (not considered a critical error) */
                        } else {
                            $YANA->report(new Log("Note: Unable to install database '$item'. Object already exists."));
                            continue;
                        }

                    } else {
                        /* else (execution finished successfully) */
                    }
                } /* end if */

            /* If a SQL file is available, always prefer using the SQL file.
             */
            } else if (!$database->importSQL($installFile)) {

                /* If import of SQL file has failed */
                if (strpos($database->getErrorMessage(), 'already exists') === false) {
                    $isSilent || $YANA->report(new Error("SQL installation fault. Unable to install database '$item'."));
                    return false;

                /* already exists (not considered a critical error) */
                } else {
                    $YANA->report(new Log("Note: Unable to install database '$item'. Object already exists."));
                    continue;
                }                

            /* If the SQL file has been imported successfully
             */
            } else {

                $YANA->report(new Log("SQL file '$installFile' has been imported."));

            } /* end if */

            /* initialize tables (store)
             */
            assert('!isset($initStmts); // Cannot redeclare var $initStmts');
            $initStmts = $database->structure->getInit();
            if (is_array($initStmts) && !empty($initStmts)) {
                assert('!isset($parser); // Cannot redeclare var $parser');
                assert('!isset($stmt); // Cannot redeclare var $stmt');
                foreach ($initStmts as $stmt)
                {
                    $parser = new DbQueryParser($database);
                    if ($parser->parseSQL($stmt)) {
                        $initialization[] = $parser;
                    }
                }
                unset($stmt, $parser);
            }
            unset($initStmts);

        } /* end foreach */

        /* initialize tables (send)
         */
        foreach ($initialization as $stmt)
        {
            if (!$stmt->sendQuery()) {
                trigger_error($stmt->db->getMessage(), E_USER_WARNING);
                return false;
            } else {
                $stmt->db->write();
            }
        }

        return true;
    }

    /**
     * synchronize FileDb with online database
     *
     * parameters taken:
     *
     * <ul>
     * <li> string database_dbms </li>
     * <li> array database_list  </li>
     * </ul>
     *
     *
     * @todo   sort table definitions by foreign keys, so referenced tables are synchronized before the references
     *
     * @access  public
     * @param   array  $ARGS array of params passed to the function
     * @return  bool
     */
    function db_sync($ARGS)
    {
        settype($ARGS, "array");
        global $YANA;
        
        @set_time_limit(500);

        if (isset($ARGS['silent']) && $ARGS['silent'] === true) {
            $isSilent = true;
        } else {
            $isSilent = false;
        } $isSilent = false;

        if (!class_exists("DB")) {
            if (!$isSilent) {
                $YANA->report( new Error('PEARDB_FAILURE') );
            }    
            return false;
        } else if (!isset($ARGS['database_dbms']) || !isset($ARGS['database_list']) || !is_array($ARGS['database_list']) || count($ARGS['database_list']) === 0) {
            return false;
        }

        foreach ($ARGS['database_list'] as $item)
        {
            /**
             * read database content
             */
            $db = new DbStream($item);
            $fileDb = new FileDb($item);

            /**
             * prepare queries
             */
            $dbQuery = new DbQuery($db);
            $dbQuery->useInheritance(false);
            $dbQuery->setType(YANA_DB_SELECT);

            foreach ($fileDb->structure->getTables() as $table)
            {
                /**
                 * select table
                 */
                $dbQuery->setTable($table);

                /**
                 * treat columns of types 'image' and 'file' as string
                 */
                $files = $fileDb->structure->getFiles($table);
                foreach ($files as $colName)
                {
                    $fileDb->structure->setType($table, $colName, 'string');
                    $db->structure->setType($table, $colName, 'string');
                }

                /* get primary key */
                $primary_key = $fileDb->structure->getPrimaryKey($table);

                if ($db->exists($table)) {
                    $db_keys = $db->get("$table.*.$primary_key");
                } else {
                    $db_keys = array();
                }
                if (!is_array($db_keys)) {
                    $db_keys = array();
                }
                if ($fileDb->exists($table)) {
                    $file_keys = $fileDb->get("$table.*.$primary_key");
                } else {
                    $file_keys = array();
                }
                if (!is_array($file_keys)) {
                    $file_keys = array();
                }

                /**
                 * synchronize: database -> file
                 */
                $i = 0;
                foreach (array_diff($db_keys, $file_keys) as $key)
                {
                    $dbQuery->setRow($key);
                    if (!$fileDb->insert("$table.$key", $db->get($dbQuery))) {
                        if (!$isSilent) {
                            trigger_error("Unable to copy value $table.$key from database to FileDB", E_USER_NOTICE);
                        } else {
                            $YANA->report( new Log("Error while trying to copy values from $table.") );
                        }
                        return false;
                    } else if ($i > 20) {
                        if ($fileDb->write()) {
                            $i = 0;
                        } else {
                            if (!$isSilent) {
                                trigger_error("Failed to commit changes to FileDB.", E_USER_NOTICE);
                            }
                            return false;
                        }
                    } else {
                        $i++;
                    }
                }
                if ($i > 0 && !$fileDb->write()) {
                    if (!$isSilent) {
                        trigger_error("Failed to commit changes to FileDB.", E_USER_NOTICE);
                    }
                    return false;
                }
                unset($i);

                /**
                 * synchronize: file -> database
                 */
                $i = 0;
                assert('!isset($diff); // Cannot redeclare var $diff');
                $diff = array_diff($file_keys, $db_keys);
                if (!empty($diff)) {
                    $fileContent = $fileDb->get($table);
                    foreach (array_diff($file_keys, $db_keys) as $key)
                    {
                        if (!$db->insert("$table.$key", $fileContent[$key])) {
                            if (!$isSilent) {
                                trigger_error("Unable to copy value $table.$key from FileDB to database", E_USER_NOTICE);
                            } else {
                                $YANA->report( new Log("Error while trying to copy values in $table to database.") );
                            }
                            return false;
                        } else if ($i > 20) {
                            if ($db->write()) {
                                $i = 0;
                            } else {
                                if (!$isSilent) {
                                    trigger_error("Failed to commit changes to Database.", E_USER_NOTICE);
                                }
                                return false;
                            }
                        } else {
                            $i++;
                        }
                        unset($fileContent[$key]);
                    }
                }
                unset($diff);
                if ($i > 0 && !$db->write()) {
                    if (!$isSilent) {
                        trigger_error("Failed to commit changes to Database.", E_USER_NOTICE);
                    }
                    return false;
                }
                unset($i);
            } /* end foreach */

        } /* end foreach */

        return true;
    }

    /**
     * backup databases
     *
     * parameters taken:
     *
     * <ul>
     * <li> string  database_dbms    the DBMS to install databases on </li>
     * <li> array   database_list     a list of databases to install </li>
     * <li> array   database_options  a list of flags: structure, data, zip </li>
     * </ul>
     *
     * @access  public
     * @param   array  $ARGS array of params passed to the function
     * @return  bool
     */
    function db_backup($ARGS)
    {
        settype($ARGS, "array");
        global $YANA;

        $useStructure = !empty($ARGS['database_options']['structure']);
        $useData = !empty($ARGS['database_options']['data']);
        $useZip = !empty($ARGS['database_options']['zip']);
        $dbms = @strtoupper($ARGS['database_target_dbms']);
        $fileContents = "";

        /* Mapping the DBMS to the SQL export function in class DbCreator */
        switch ($dbms)
        {
            case 'DB2':
                $methodName  = 'createDB2';
            break;
            case 'ACCESS':
                $methodName  = 'createMSAccess';
            break;
            case 'MSSQL':
                $methodName  = 'createMSSQL';
            break;
            case 'MYSQL':
            case 'MYSQLI':
                $dbms = 'MySQL';
                $methodName  = 'createMySQL';
            break;
            case 'OCI8':
                $methodName  = 'createOracleDB';
            break;
            case 'PGSQL':
                $dbms = 'PostGreSQL';
                $methodName  = 'createPostgreSQL';
            break;
            case 'DBASE':
            case 'FBSQL':
            case 'IBASE':
            case 'IFX':
            case 'SYBASE':
            default:
                $YANA->report( new Alert('FIELD_INVALID', array( 'FIELD' => 'DBMS=' . $dbms ) ));
                return false;
            break;
        }

        foreach ($ARGS['database_list'] as $dbName)
        {
            if (is_string($dbName)) {
                $db = $YANA->connect($dbName);
                $dbc = new DbExtractor($db);
                $arrayOfStmts = $dbc->$methodName($useStructure, $useData);
                $fileContents .= implode("\n", $arrayOfStmts) . "\n";
            }
        }

        $filename = strtolower(preg_replace('/\W/', '_', $dbms) . '.sql');
        if (!empty($fileContents)) {
            header("Pragma: no-cache");
            if ($useZip) {
                $fileContents = gzencode($fileContents, 9);
                $filename .= '.gz';
                header("Content-type: text/plain");
                header("Content-Disposition: attachment; filename=${filename}");
                header("Content-Length: " . strlen($fileContents));
            } else {
                header("Content-type: text/plain");
                header("Content-Disposition: attachment; filename=${filename}");
                header("Content-Length: " . strlen($fileContents));
            }
            exit($fileContents);
            return true;
        } else {
            $YANA->report(new Error("FILE_NOT_CREATED", array('FILE' => $filename)));
            return false;
        }
    }

    /**
     * update database setup
     *
     * parameters taken:
     *
     * <ul>
     * <li> string database_active   </li>
     * <li> string database_dbms     </li>
     * <li> string database_host     </li>
     * <li> string database_port     </li>
     * <li> string database_password </li>
     * <li> string database_name     </li>
     * <li> string database_prefix   </li>
     * </ul>
     *
     * @access public
     * @param array $ARGS array of params passed to the function
     * @return boolean
     */
    function set_db_configuration($ARGS)
    {
        settype($ARGS, "array");
        global $YANA;

        /* update database settings */
        if (isset($ARGS['database_dbms'])) {

            /**
             * 1) Test if connection is available
             */
            if ($ARGS['database_active'] === 'true') {
                /* check input data */
                $dsn = array('DBMS'     => $ARGS['database_dbms'],
                             'HOST'     => $ARGS['database_host'],
                             'PORT'     => $ARGS['database_port'],
                             'USERNAME' => stripslashes($ARGS['database_user']),
                             'PASSWORD' => stripslashes($ARGS['database_password']),
                             'DATABASE' => stripslashes($ARGS['database_name']));

                $test = dbServer::isAvailable($dsn);
                if ($test !== true) {
                    $YANA->report(new Error('Unable to establish connection to database server. Check your input please!'));
                    return false;
                } else {
                    /* all fine - proceed */
                }
            } else {
                /**
                 * A connection to FileDB should always be available,
                 * so there is no need to check that.
                 * Instead we assume this test is always "true".
                 */
            }

            /**
             * 2) Contact database server
             *
             * Install and synchronize the database with the current content of FileDb.
             *
             * This only needs to be done when a user activates / deactivates database support.
             * If the "database_active" setting is unchanged, then this may be skipped
             */
            if (isset($ARGS['database_active'])) {
                if (YANA_DATABASE_ACTIVE && $ARGS['database_active'] === 'true') {
                    /* continue without changes */
                } else if (!YANA_DATABASE_ACTIVE && $ARGS['database_active'] === 'false') {
                    /* continue without changes */
                } else {
                    $ARGS['silent'] = true;
                    if ($ARGS['database_active'] === 'true') {
                        if (!empty($ARGS['database_autoinstall']) && $ARGS['database_autoinstall'] === 'true') {
                            $test = $this->db_install($ARGS);
                            if ($test !== true) {
                                $YANA->report(new Error('Installation of tables failed. (Do tables already exist?)'));
                                $YANA->report(new Log('Notice: installation of tables failed. (May be tables already exist?)'));
                            } else {
                                /* success - proceed to next step */
                            }
                        } else {
                            /* continue without changes */
                        }
                    } else {
                        /* continue (FileDB does not need a database installation) */
                    }
                    if (!empty($ARGS['database_autosync']) && $ARGS['database_autosync'] === 'true') {
                        $test = $this->db_sync($ARGS);
                        if ($test !== true) {
                            $YANA->report(new Error('500'));
                            $YANA->report(new Log('Unable to install tables of plugin "user" with the choosen dbms. Operation aborted.'));
                            return false;
                        } else {
                            /* success - proceed to next step */
                        } /* end if */
                    } else {
                        /* continue without changes */
                    }
                } /* end if */
            } else {
                /* continue without changes */
            } /* end if */

            /**
             * 3) Write changes to file
             */
            $text = "<?php

if (!defined('YANA_DATABASE_ACTIVE'))   define('YANA_DATABASE_ACTIVE', ".     ( (isset($ARGS['database_active']))   ? $ARGS['database_active']   : YANA_DATABASE_ACTIVE )   .");
if (!defined('YANA_DATABASE_DBMS'))     define('YANA_DATABASE_DBMS', \"".     ( (isset($ARGS['database_dbms']))     ? $ARGS['database_dbms']     : YANA_DATABASE_DBMS )     ."\");
if (!defined('YANA_DATABASE_HOST'))     define('YANA_DATABASE_HOST', \"".     ( (isset($ARGS['database_host']))     ? $ARGS['database_host']     : YANA_DATABASE_HOST )     ."\");
if (!defined('YANA_DATABASE_PORT'))     define('YANA_DATABASE_PORT', \"".     ( (isset($ARGS['database_port']))     ? $ARGS['database_port']     : YANA_DATABASE_PORT )     ."\");
if (!defined('YANA_DATABASE_USER'))     define('YANA_DATABASE_USER', \"".     ( (isset($ARGS['database_user']))     ? $ARGS['database_user']     : YANA_DATABASE_USER )     ."\");
if (!defined('YANA_DATABASE_PASSWORD')) define('YANA_DATABASE_PASSWORD', \"". ( (isset($ARGS['database_password'])) ? $ARGS['database_password'] : YANA_DATABASE_PASSWORD ) ."\");
if (!defined('YANA_DATABASE_PREFIX'))   define('YANA_DATABASE_PREFIX', \"".   ( (isset($ARGS['database_prefix']))   ? $ARGS['database_prefix']   : YANA_DATABASE_PREFIX )   ."\");
if (!defined('YANA_DATABASE_NAME'))     define('YANA_DATABASE_NAME', \"".     ( (isset($ARGS['database_name']))     ? $ARGS['database_name']     : YANA_DATABASE_NAME )     ."\");

?>";
            if (!$this->file->exists()) {
                $this->file->create();
            }
            $this->file->read();
            $this->file->remove();
            $this->file->insert($text);
            $this->file->failSafeWrite();

        } else {
            $YANA->report(new Error('INPUT_INVALID'));
            return false;
        } /* end if */

        return true;
    }

}
// Copyright  2009  Thomas Meyer  http://www.yanaframework.net
?>