<?php
/**
 * Plugin for Yana PHP-Framework
 *
 * Software:        Yana PHP-Framework 3.0.0
 * Author:          Dipl. Inf. Thomas Meyer
 * Homepage:        http://www.yanaframework.net
 * Copyright:       (c) 2009  Thomas Meyer
 * License:         GNU GPL  http://www.gnu.org/licenses/
 *
 * This program: can be redistributed and/or modified under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/.
 *
 * You may use this software for free. In case you like it, or want to
 * use it for a commercial purpose, please consider making a donation.
 * http://yanaframework.net/donate.php
 *
 * This notice MAY NOT be removed.
 *
 *
 * @author     Thomas Meyer
 * @license    http://www.gnu.org/licenses/gpl.txt
 * @package    yana
 * @subpackage plugins
 */

/**
 * Database tools
 *
 * This implements a several database modeling and maintenance utilities
 *
 * @access     public
 * @package    yana
 * @subpackage plugins
 */
class plugin_db_tools extends Plugin
{

    /**
     * List of exportable DBMS ids and names
     *
     * @access  private
     * @var     array
     * @ignore
     */
    var $listOfExportableDBMS = array(
        'mysql'  => 'MySQL',
        'db2'    => 'DB2',
        'access' => 'MS-Access',
        'mssql'  => 'MS-SQL',
        'oci8'   => 'Oracle',
        'pgsql'  => 'PostgreSQL',
    );

    /**
     * Default event handler
     *
     * returns bool(true) on success and bool(false) on error
     *
     * @access  public
     * @return  bool
     * @param   string  $event  name of the called event in lower-case
     * @param   array   $ARGS   array of arguments passed to the function
     * @ignore
     */
    function _default($event, $ARGS)
    {
        global $YANA;
        /* do nothing */
        return true;
    }

    /**
     * Create form
     *
     * Type: read
     * Permission: 100
     * Templates:  db_tools
     *
     * @param array $ARGS array of params passed to the function
     * @return bool
     * @ignore
     */
    function db_tools_config_import ($ARGS)
    {
        global $YANA;
        /* do nothing */
        return true;
    }

    /**
     * Create form
     *
     * Type: read
     * Permission: 100
     * Templates:  db_tools
     *
     * @param array $ARGS array of params passed to the function
     * @return bool
     * @ignore
     */
    function db_tools_config_export ($ARGS)
    {
        global $YANA;
        $YANA->setVar('SELECTED_DBMS', YANA_DATABASE_DBMS);
        $YANA->setVar('LIST_OF_EXPORTABLE_DBMS', $this->listOfExportableDBMS);
        $DATABASE_LIST = dirlist($YANA->getVar("DBDIR"), '*.config');
        foreach ($DATABASE_LIST as $key => $item)
        {
            $DATABASE_LIST[$key] = preg_replace("/\.\w*$/", "", $item);
        }
        $YANA->setVar("DATABASE_LIST", $DATABASE_LIST);
        return true;
    }

    /**
     * Reverse-engineer database
     *
     * Type: default
     * Permission: 100
     * Templates: MESSAGE
     *
     * @param   array  $ARGS  array of params passed to the function
     * @return  bool
     * @ignore
     */
    function db_tools_write_config ($ARGS)
    {
        global $YANA;

        $db = new DbStream();
        $structure = $db->structure->getStructure();
        if (!is_array($structure)) {
            return false;
        } else {
            $structure = SML::encode($structure);
        }
        $filename = 'database.config';
        if (!empty($structure)) {
            header("Content-Disposition: attachment; filename=${filename}");
            header("Content-type: text/plain");
            header("Content-Length: " . strlen($structure));
            exit($structure);
            return true;
        } else {
            $YANA->report(new Error("FILE_NOT_CREATED", array('FILE' => $filename)));
            return false;
        }
    }

    /**
     * export database content to xml
     *
     * Type: default
     * Permission: 100
     * Templates: MESSAGE
     *
     * @param   array  $ARGS  array of params passed to the function
     * @return  bool
     * @ignore
     */
    function db_tools_exportxml ($ARGS)
    {
        global $YANA;

        if (isset($ARGS['database_list']) && is_array($ARGS['database_list']) && !empty($ARGS['database_list'])) {
            $dbList = array_values($ARGS['database_list']);

        } else {
            $YANA->report(new Warning("INPUT_INVALID"));
            return false;

        }

        $xml = DbExtractor::createXML(true, null, $dbList);
        $filename = 'database.xml';
        if (!empty($xml)) {
            header("Content-Disposition: attachment; filename=${filename}");
            header('Content-Type: application/xml');
            header("Content-Length: " . strlen($xml));
            exit($xml);
            return true;

        } else {
            $YANA->report(new Error("FILE_NOT_CREATED", array('FILE' => $filename)));
            return false;
        }
    }

    /**
     * Import and convert MDB2 schema file to Yana database structure
     *
     * Type: default
     * Permission: 100
     * Templates: MESSAGE
     *
     * @param   array  $ARGS  array of params passed to the function
     * @return  bool
     * @ignore
     */
    function db_tools_importmdb2 ($ARGS)
    {
        if (!empty($_FILES['mdb2']['tmp_name'])) {
            if (!($structure = DbExtractor::importMDB2($_FILES['mdb2']['tmp_name']))) {
                return false;
            } else {
                $structure = $structure->getStructure();
            }
            if (!is_array($structure)) {
                return false;
            } else {
                $structure = SML::encode($structure);
            }
            $filename = 'database.config';
            if (!empty($structure)) {
                header("Content-Disposition: attachment; filename=${filename}");
                header("Content-type: text/plain");
                header("Content-Length: " . strlen($structure));
                exit($structure);
                return true;
            } else {
                $YANA->report(new Error("FILE_NOT_CREATED", array('FILE' => $filename)));
                return false;
            }
        } else {
            return false;
        }
    }

    /**
     * Import and convert MDB2 schema file to Yana database structure
     *
     * Type: default
     * Permission: 100
     * Templates: MESSAGE
     *
     * @param   array  $ARGS  array of params passed to the function
     * @return  bool
     * @ignore
     */
    function db_tools_importdbdesigner4 ($ARGS)
    {
        if (!empty($_FILES['dbdesigner4']['tmp_name'])) {
            if (!($structure = DbExtractor::importDbDesigner4($_FILES['dbdesigner4']['tmp_name']))) {
                return false;
            } else {
                $structure = $structure->getStructure();
            }
            if (!is_array($structure)) {
                return false;
            } else {
                $structure = SML::encode($structure);
            }
            $filename = 'database.config';
            if (!empty($structure)) {
                header("Content-Disposition: attachment; filename=${filename}");
                header("Content-type: text/plain");
                header("Content-Length: " . strlen($structure));
                exit($structure);
                return true;
            } else {
                $YANA->report(new Error("FILE_NOT_CREATED", array('FILE' => $filename)));
                return false;
            }
        } else {
            return false;
        }
    }

    /**
     * Forward-engineer database
     *
     * Type: default
     * Permission: 100
     * Templates: NULL
     *
     * @param   array  $ARGS  array of params passed to the function
     * @return  bool
     * @ignore
     */
    function db_tools_exportsql ($ARGS)
    {
        global $YANA;

        $fileContents = "";

        foreach ($ARGS['database_list'] as $dbName)
        {
            if (is_string($dbName)) {

                /* Mapping the DBMS to the SQL export function in class DbCreator */
                switch ($ARGS['dbms'])
                {
                    case 'db2':
                        $methodName  = 'createDB2';
                    break;
                    case 'access':
                        $methodName  = 'createMSAccess';
                    break;
                    case 'mssql':
                        $methodName  = 'createMSSQL';
                    break;
                    case 'mysql':
                    case 'mysqli':
                        $methodName  = 'createMySQL';
                    break;
                    case 'oci8':
                        $methodName  = 'createOracleDB';
                    break;
                    case 'pgsql':
                        $methodName  = 'createPostgreSQL';
                    break;
                    case 'dbase':
                    case 'fbsql':
                    case 'ibase':
                    case 'ifx':
                    case 'sybase':
                    default:
                        $YANA->report( new Alert('FIELD_INVALID', array( 'FIELD' => 'DBMS=' . $ARGS['dbms'] ) ));
                        return false;
                    break;
                }
                $dbc = new DbCreator( new DbStructure($dbName) );
                $arrayOfStmts = $dbc->$methodName();
                $fileContents .= implode("\n", $arrayOfStmts) . "\n";
            }
        }

        $filename = strtolower(preg_replace('/\W/', '_', $ARGS['dbms']) . '.sql');
        if (!empty($fileContents)) {
            header("Content-Disposition: attachment; filename=${filename}");
            header("Content-type: text/plain");
            header("Content-Length: " . strlen($fileContents));
            exit($fileContents);
            return true;
        } else {
            $YANA->report(new Error("FILE_NOT_CREATED", array('FILE' => $filename)));
            return false;
        }
    }
}
// Copyright  2009  Thomas Meyer  http://www.yanaframework.net
?>