<?php
/**
 * Plugin for Yana PHP-Framework
 *
 * Software:        Yana PHP-Framework 3.0.0
 * Author:          Dipl. Inf. Thomas Meyer
 * Homepage:        http://www.yanaframework.net
 * Copyright:       (c) 2009  Thomas Meyer
 * License:         GNU GPL  http://www.gnu.org/licenses/
 *
 * This program: can be redistributed and/or modified under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/.
 *
 * You may use this software for free. In case you like it, or want to
 * use it for a commercial purpose, please consider making a donation.
 * http://yanaframework.net/donate.php
 *
 * This notice MAY NOT be removed.
 *
 *
 * @author     Thomas Meyer
 * @license    http://www.gnu.org/licenses/gpl.txt
 * @package    yana
 * @subpackage plugins
 */

/**
 * Self-diagnosis plug-in
 *
 * Runs a self-test on the current installation of the Yana Framework
 * and displayes the results.
 * It informs about found problems and provides hints on how to solve them.
 *
 * @access     public
 * @package    yana
 * @subpackage plugins
 */
class plugin_diagnostics extends Plugin
{
    /**
     * run system diagnostics
     *
     * This function takes 1 argument: 'target', with the value 'detailed'.
     * If set, additional information is provided.
     *
     * @access  public
     * @param   array  $ARGS  user form data
     * @return  bool
     */
    function test ($ARGS)
    {
        assert('is_array($ARGS); // Wrong argument type argument 1. Array expected.');
        $details = !empty($ARGS['target']) && $ARGS['target'] === 'detailed';
        assert('is_bool($details); // Unexpected result $details. Boolean expected.');
        unset($ARGS);
        
        global $YANA;

        /*
         * IMPORTANT NOTE:
         * DON'T TOUCH IT, since it might be used as input in other apps for aditional error-analysis.
         * This is highly usefull, since not all errors that COULD be extracted from the information
         * provided in this report, currently ARE reported as errors by this function.
         * This comes 'cuz it simply can't "know" everything. ;-)
         *
         * E.g. if there is a known problem with a certain version of a plugin in a wild combination
         * with a number of others, this won't check. But an additional analysis with another app that
         * "knows" this behaviour, might find this error. No matter what version of the framework you
         * are currently running. BUT ONLY IF the output of this function is reliable and remains
         * the same from version to version.
         * If the output is changed in any way, you never know which other applications you might break!
         *
         * TODO:
         * might want to create an alternative xml-output for this function to better assist error analysis
         */

        echo "<?xml version=\"1.0\"?>\n" .
        "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n\n" .
        "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">\n\n" .
        "<head>\n\t" .
        "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\" />\n\t" .
        "<meta name=\"robots\" content=\"NOINDEX, NOFOLLOW\" />\n\t" .
        "<title>Self-Test Logfile</title>\n\t" .
        "<style type=\"text/css\">" .
        "div.error, div.success, div.notice, divstatus { margin-top: 10px; margin-bottom: 10px; display: block; }" .
        ".instruction .description { text-align: justify; }" .
        ".error { color: #ff0000; }" .
        ".success { color: #00a000; }" .
        ".notice { color: #0000a0; }" .
        ".status { color: #000000; } " .
        ".instruction { border: 1px solid #888; margin: 10px; padding: 10px; color: #000000; background: #f8f8f8; } " .
        ".description { color: #808080; } " .
        ".label { font-weight: bold; margin-right: 10px; } " .
        ".option { margin-left: 30px; } " .
        "body { background: #ffffff; color: #000000; font-family: \"Times New Roman\", serif; } " .
        'h1 { padding: 5px; font-family: "Arial", sans-serif; background: #dde5fa; color: #000000; } ' .
        'h2 { padding: 5px; font-family: "Arial", sans-serif; background: #eeeeee; color: #000000; } ' .
        'h3 { padding: 5px; font-size: 14px; font-family: "Arial", sans-serif; background: #d8e0e8; color: #000000; }</style>' .
        "\n</head>\n\n<body>

        <h1>Self-Test Logfile</h1>

        <div id=\"documentation\">

        <div class=\"instruction\">
        <h2>Anleitung</h2>
        <p class=\"description\">Dieses Skript fhrt eine Reihe von Tests zur Selbstdiagnose durch, um festzustellen,
           ob Ihr Programm korrekt installiert wurde. Sie knnen diese Log-Datei selbst auf Fehlermeldungen untersuchen - oder aber,
           falls das Programm dennoch nicht funktioniert, knnen Sie mir auch eine E-Mail senden, um Hilfe bei der Lsung
           Ihres Problems zu erhalten. Wenn Sie Fragen via Mail stellen, hngen Sie bitte dieses Diagnoseprotokoll an Ihre Mail.
           Viele typische Fehler lassen sich bereits aus diesem Protokoll erkennen und Sie erhalten schneller eine Antwort.</p>
        <ul class=\"description\">
           <li>Falls ein Test fehlschlgt, werden <span class=\"error\">Fehlermeldungen</span> rot markiert.</li>
           <li>Ist ein Test erfolgreich, werden <span class=\"success\">Besttigungsmeldungen</span> grn markiert.</li>
           <li>Alle <span class=\"status\">normalen Status-Informationen</span> werden schwarz dargestellt.</li>
        </ul>
        <p class=\"description\">Sollten Sie rote Zeilen finden: prfen Sie bitte Ihre Konfiguration auf die beschriebenen Fehler.<br />
           Bitte beachten Sie: dieses Programm erkennt zwar einige der <i>hufigsten Fehler</i>,
           aber es kann nicht alle mglichen Fehlerquellen entdecken.
        </p>

        </div>

        <div class=\"instruction\">

        <h2>Instructions</h2>
        <p class=\"description\">This self-diagnosis script performs a number of tests to tell, if your program
           has been installed correctly. You may check this log-file yourself for error-messages - or, if it still does not work,
           email me to get help fixing your problem. By doing so, please include a copy of this diagnosis protocol with your mail.
           Many typical problems may be recognized by reviewing this protocol and you will possibly get an answer for your problem
           a little faster.
        </p>
        <ul class=\"description\">
           <li>If a test fails, <span class=\"error\">failure-messages</span> are marked red.</li>
           <li>If a test succeeds, <span class=\"success\">success-messages</span> are marked green.</li>
           <li>All <span class=\"status\">normal status-information</span> is printed black.</li>
        </ul>
        <p class=\"description\">If you find any red lines: check your configuration for these errors.<br />
           Please note: this program detects some of the <i>most common errors</i>, but it does not find all possible problems.
        </p>

        </div>

        <p><a href=" . SmartUtility::href('action=' . __FUNCTION__ . (($details) ? '' : '&target=detailed')) . ">" .
        (($details) ? 'show less details' : 'show more details') . "</a></p>

        <div class=\"label\">Program is performing self-test" .
        ( ($YANA->getVar('id')) ? " on profile '" . $YANA->getVar('id') . "'" : "" ) .
        " now ...</div>

        </div>

        <ul class=\"status\">
            <li>installed version of Yana Framework is: " . YANA_VERSION . "</li>
            <li>installed version of PHP is: " . PHP_VERSION . "</li>
            <li>current server time index is: " . date("r", time()) . "</li>
        </ul>

        <h1>\tTESTING INSTALLATION:</h1>\n";

        echo '<div id="init">';
        /**
         * 1) Check for availability of PEAR-DB
         */
        @include_once("DB.php");
        if (!class_exists("DB")) {
            echo '<div class="error"><span class="label">Error:</span>' .
            '<span class="description">PHP PEAR-DB module not found. Database plugins require PEAR-DB and will not run unless you install it.</span>' .
            "</div>\n";
        } else {
            echo '<div class="success"><span class="label">okay:</span>' .
            '<span class="description">PHP PEAR-DB found (required to run database plugins)</span>' .
            "</div>\n";
        }

        /**
         * 2) Check for availability of the framework's primary controller
         */
        if (!isset($GLOBALS['YANA'])) {
            echo '<div class="error"><span class="label">Error:</span>' .
            '<span class="description">Unable to access Yana instance. The framework will not run properly. Please reinstall the application.</span>' .
            "</div>\n";
        }

        /**
         * 3) Check availability of configuration file and configuration directory
         */
        if (YANA_CDROM === true) {
            if (!is_writeable(YANA_CDROM_DIR)) {
                echo '<div class="error"><span class="label">Error:</span>' .
                '<span class="description">Temporary directory ' . YANA_CDROM_DIR . ' is not writeable. Set access rights for directory &quot;' .
                YANA_CDROM_DIR . '&quot; to 777, including all subdirectories and files.</span>' .
                "</div>\n";
            }
        } else {
            if (!is_writeable($YANA->getVar('CONFIGDIR'))) {
                echo '<div class="error"><span class="label">Error:</span>' .
                '<span class="description">Configuration directory is not writeable. Set access rights for directory &quot;' .
                $YANA->getVar('CONFIGDIR') . '&quot; to 777, including all subdirectories and files.</span>' .
                "</div>\n";
            }
            if (!is_writeable($YANA->getVar('TEMPDIR'))) {
                echo '<div class="error"><span class="label">Error:</span>' .
                '<span class="description">Directory for temporary files is not writeable. Set access rights for directory &quot;' .
                $YANA->getVar('TEMPDIR') . '&quot; to 777, including all subdirectories and files.</span>' .
                "</div>\n";
            }
        }
        echo '</div>';

        echo '<div id="config">';
        echo "<h2>\tTESTING CONFIGURATION DIRECTORY:</h2>\n";

        /**
         * 4) Check availability contents of virtual directory
         */
        $drive = $YANA->registry;
        if (!$drive->exists()) {
            die ('<div class="error"><span class="label">Critical error:</span>' .
            '<span class="description">Configuration file &quot;' . $drive->getPath() . '&quot; cannot be found.</div>' .
            '<hr /><div class="status">Test aborted due to a critical error.</div>');
        }

        $drive->read();
        $test = $drive->getReport();
        unset($drive);
        if (!$test) {
            echo '<div class="success"><span class="label">No problems found.</span></div>' . "\n";
        } else {
            echo '<div class="error"><span class="label">Your configuration contains the following errors:</span></div>' . $test;
        }
        echo '</div>';

        /**
         * 5) check system integrity
         */
        if ($details) {
            echo '<div id="system">';
            echo "<h2>\tSYSTEM-INTEGRITY CHECK:</h2>\n";
            echo "<pre>";
            if (is_dir('manual')) {
                print '<div class="notice"><span class="label">Notice:</span>' .
                '<span class="description">You do not need to copy the directory &quot;manual&quot; to your website.' .
                "\n\tIt is not required to run the program. You might want to remove it to safe space.</span></div>\n";
            }
            $root = dirlist(".", '.php');
            assert('!isset($i); /* Cannot redeclare variable $i */');
            for ($i = 0; $i < count($root); $i++)
            {
                if ($root[$i] != 'index.php' && $root[$i] != 'library.php' && $root[$i] != 'cli.php') {
                    if (strpos($root[$i],'update') !== false) {
                        echo '<div class="error"><span class="label">Warning:</span>'.
                        '<span class="description">The file &quot;./' . $root[$i] . '&quot; was found.' .
                        "\n\tTelling by the name, this might be an update script that you forgot to delete.\n\t" .
                        "For security reasons you should delete this file to prevent missuse by any third-party.</span></div>";
                    } else {
                        echo '<div class="error"><span class="label">Warning:</span>'.
                        '<span class="description">Unexpected file &quot;./' . $root[$i] . '&quot; found.' .
                        "\n\tIf you did'nt place this file here, it might be the result of an hijacking attempt.\n\t" .
                        "You should consider removing this file.</span></div>";
                    }
                } else {
                    echo "SYSTEM[./".$root[$i]."]";
                    assert('!isset($j); /* Cannot redeclare variable $j */');
                    for ($j = 0; $j < (40 - strlen($root[$i])); $j++)
                    {
                        echo " ";
                    }
                    unset($j);
                    echo '= '. md5(file_get_contents($root[$i]))."#".crc32(file_get_contents($root[$i])) . "\n";
                }
            }
            unset($i);
            $root = dirlist("./includes/", '.php');
            assert('!isset($i); /* Cannot redeclare variable $i */');
            for ($i = 0; $i < count($root); $i++)
            {
                echo "SYSTEM[./includes/".$root[$i]."]";
                assert('!isset($j); /* Cannot redeclare variable $j */');
                for ($j = 0; $j < (31 - strlen($root[$i])); $j++)
                {
                    echo " ";
                }
                unset($j);
                echo '= ' . md5(file_get_contents("includes/".$root[$i]))."#".crc32(file_get_contents("includes/".$root[$i])) . "\n";
            } /* end for */
            unset($i);
            echo '</pre>';
            echo '</div>';

            /**
             * 6) check accessability of configuration files
             */
            echo '<div id="files">';
            echo "<h2>\tAUTOMOUNTING ALL FILES:</h2>\n";

            assert('!isset($key) && !isset($element); /* cannot redeclare variable */');
            foreach ($YANA->registry->drive as $key => $element)
            {
                if ($element->mount() && $element->mountpoint->exists()) {
                    if (is_readable($element->mountpoint->getPath())) {
                        echo '<div class="success"><span class="label">okay</span>' .
                        '<span class="description">&quot;' . $element->mountpoint->getPath() . '&quot; is available</span>' .
                        "</div>\n";
                    } else {
                        echo '<div class="error"><span class="label">Error</span>' .
                        '<span class="description">&quot;' . $element->mountpoint->getPath() . '&quot; is not readable</span>' .
                        "</div>\n";
                    }
                } else {
                    echo '<div class="notice"><span class="label">Notice:</span>' .
                    '<span class="description">&quot;' . $element->mountpoint->getPath() . '&quot; is not available.</span>' .
                    "</div>\n";
                }
            } /* end foreach */
            unset($key, $element);
            echo '</div>';
        }

        /**
         * 7) check accessability of plugin files
         */
        echo '<div id="plugin">';

        echo "<h2>\tTESTING PLUGIN CONFIGURATION:</h2>\n";
        $YANA->plugins->mountAll();

        assert('!isset($id);    /* Cannot redeclare variable $id */');
        assert('!isset($drive); /* Cannot redeclare variable $drive */');
        foreach ($YANA->plugins->drive as $id => $drive)
        {
            if (!$drive->exists()) {
                die('<div class="error"><span class="label">Critical error:</span>' .
                '<span class="description">Configuration file &quot;' . $drive->getPath() . '&quot; cannot be found.</div>' .
                '<hr /><div class="status">Test aborted due to a critical error.</div>');
            } else {
                $drive->read();
                echo '<div class="option">' . "\n";
                echo "<h3>Testing Plugin &quot;${id}&quot;</h3>\n";
                $test = $drive->getReport();
                if (!$test) {
                    echo '<div class="success"><span class="label">No problems found.</span></div>' . "\n";
                } else {
                    echo '<div class="error"><span class="label">Your configuration contains the following errors:</span></div>' . $test;
                }
                echo "</div>\n";
            }
        } /* end foreach */
        unset($id, $drive);

        echo '</div>';

        /**
         * 8) check configuration on safe-mode
         */
        echo "<h1>\tTESTING BACKUP-CONFIGURATION:</h1>\n";

        /**
         * 8.1) check profile settings
         */
        echo '<div id="profile">';

        echo "<h2>\tCHECKING PROFILE FOR ERRORS:</h2>\n";
        if (count($YANA->registry->drive['config']->mountpoint->get("PROFILE")) == 0) {
            echo '<div class="error"><span class="label">Error:</span>' .
            '<span class="description">Configuration is not available.</span>' .
            "</div>\n";
        } else {
            if ($details) {
                echo '<div class="label">Performing deep-scan on file &quot;'.$YANA->registry->files['config']->getPath()."&quot;:</div>\n";
                assert('!isset($key) && !isset($element); /* cannot redeclare variable */');
                foreach ($YANA->registry->drive['config']->mountpoint->get("PROFILE") as $key => $element)
                {
                    echo "$key = ".gettype($element)."<br />\n";
                    if (is_array($element)) {
                        echo '<div class="option">' . "\n";
                        assert('!isset($a); /* Cannot redeclare variable $a */');
                        assert('!isset($b); /* Cannot redeclare variable $b */');
                        foreach ($element as $a => $b)
                        {
                            echo "${a} = " . gettype($b) . "<br />\n";
                        }
                        unset($a, $b);
                        echo "</div>\n";
                    }
                } /* end foreach */
                unset($key, $element);
            }
            echo '<div class="success"><span class="label">No problems found:</span>' .
            '<span class="description">Profile settings for file &quot;' . $YANA->registry->files['config']->getPath() .
            '&quot; are available.</span>' .
            "</div>\n";
        }

        echo '</div>';

        /**
         * 8.2) check availability of skin- and language files
         */
        echo '<div id="skin_language">';

        echo "<h2>\tCHECKING SKIN- AND LANGUAGE-SETTINGS:</h2>\n";
        $test = $YANA->skin->getReport();

        echo '<div id="skin">';

        if (!empty($test)) {
            echo $test;
            echo '<div class="error"><span class="label">Warning:</span>' .
            '<span class="description">Some errors found. This skin might either be corrupt or it was not installed correctly.</span>' .
            "</div>\n";
        } else {
            echo '<div class="success"><span class="label">No problems found:</span>' .
            '<span class="description">Skin file &quot;' . $YANA->registry->files['skin']->getPath() .
            '&quot; seems to be installed correctly.</span>' .
            "</div>\n";
        }
        if (count($YANA->registry->drive['skin']->mountpoint->get("*"))==0) {
            echo '<div class="error"><span class="label">Error:</span>' .
            '<span class="description">No skin file available.</span>' .
            "</div>\n";
        }

        echo '</div>';
        echo '<div id="language">';

        if (count($YANA->registry->drive['language']->mountpoint->get("*"))==0) {
            echo '<div class="error"><span class="label">Error:</span>' .
            '<span class="description">No language file available. Please check if this filename is correct: ' .
            '&quot;' . $YANA->registry->files['language']->getPath() . '&quot;</span>' .
            "</div>\n";
        }

        echo '</div>';
        echo '</div>';

        /**
         * 9) check availability of skin- and language files
         */
        echo '<div id="plugin_temlates">';

        echo "<h2>\tCHECKING PLUGIN-TEMPLATES:</h2>\n";
        $test = $YANA->plugins->getReport();
        if (!empty($test)) {
            echo $test;
            echo '<div class="error"><span class="label">Warning:</span>' .
            '<span class="description">Some installed plugins might either be corrupt or were not installed correctly.</span>' .
            "</div>\n";
        } else {
            echo '<div class="success"><span class="label">No problems found:</span>' .
            '<span class="description">All plugin templates seem to be installed correctly.</span>' .
            "</div>\n";
        }

        echo '</div>';

        /**
         * 10)get a list of all emot-icons (for your information only)
         */
        echo '<div id="emoticons">';

        echo "<h2>\tLIST ALL SMILIES:</h2>\n";
        SmartUtility::loadSmilies();
        $smilies = $YANA->getVar('SMILIES');
        if (count($smilies)==0) {
            echo '<div class="error"><span class="label">Warning:</span>' .
            '<span class="description">No Icons found. ' .
            'Please check if the given directory is correct: &quot;' . $YANA->registry->files['SMILE']->getPath() . '&quot;.</span>' .
            "</div>\n";
        } else {
            echo '<div class="label">&bull; <span id="icon_count">' .
            count($smilies) . "</span> Icons found in directory &quot;" . $YANA->registry->files['SMILE']->getPath() . "&quot;.</div>\n";

            echo '<div id="list_of_icons">';
            for ($i = 0; $i < count($smilies) && !empty($details); $i++)
            {
                echo '<span class="icon_name">' . $smilies[$i] . '</span>, ';
                if (($i + 1) % 5 == 0) {
                    echo "<br />\n";
                }
            }
            echo '</div>';

            echo '<div class="success"><span class="label">No problems found:</span>' .
            '<span class="description">Directory setting &quot;' . $YANA->registry->files['SMILE']->getPath() .
            '&quot; seems to be correct.</span>' .
            "</div>\n";
        }
        echo '</div>';

        echo '<hr /><div class="status">Self-Test ended at: ' . date("d.m.Y H:i:s", time()) . "</div>\n</body>\n\n</html>";
        exit(0);
    }
}
// Copyright  2009  Thomas Meyer  http://www.yanaframework.net
?>