<?php
/**
 * Plugin for Yana PHP-Framework
 *
 * Software:        Yana PHP-Framework 3.0.0
 * Author:          Dipl. Inf. Thomas Meyer
 * Homepage:        http://www.yanaframework.net
 * Copyright:       (c) 2009  Thomas Meyer
 * License:         GNU GPL  http://www.gnu.org/licenses/
 *
 * This program: can be redistributed and/or modified under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/.
 *
 * You may use this software for free. In case you like it, or want to
 * use it for a commercial purpose, please consider making a donation.
 * http://yanaframework.net/donate.php
 *
 * This notice MAY NOT be removed.
 *
 *
 * @author     Thomas Meyer
 * @license    http://www.gnu.org/licenses/gpl.txt
 * @package    yana
 * @subpackage plugins
 */

/**
 * IP-Blocker utility
 *
 * Security plugin that automatically blocks
 * banned IPs via configureable black- and whitelists.
 *
 * @access     public
 * @package    yana
 * @subpackage plugins
 */
class plugin_ipblock extends Plugin
{

    /**#@+
     * @access  private
     * @var     blockfile
     * @ignore
     */

    var $whitelist = false;
    var $blacklist = false;

    /**#@-*/

    /**
     * Constructor
     *
     * @access  public
     * @param   string  $name a name for this plugin
     * @ignore
     */
    function plugin_ipblock($name)
    {
        settype($name,"string");
        $this->name = $name;
        global $YANA;
        $this->whitelist = $YANA->plugins->get($this->name, "IPWHITELIST");
        $this->blacklist = $YANA->plugins->get($this->name, "IPBLACKLIST");
        $dir             = $YANA->plugins->get($this->name, "BLOCKDIR");

        if (!$dir->exists() && !$dir->create(777)) {
            $YANA->report( new Log("Unable to create directory", $dir->getPath()) );
            $YANA->report( new Error("500"));
        }
    }

    /**
     * check user
     *
     * Checks if the user's IP is black-listed and if so, produces an error message and returns false.
     *
     * returns bool(true) on success and bool(false) on error
     *
     * @access  public
     * @return  bool
     * @param   string  $event  name of the called event in lower-case
     * @param   array   $ARGS   array of arguments passed to the function
     * @ignore
     */
    function _default($event, $ARGS)
    {
        global $YANA;

        if ($this->blacklist && $this->whitelist && $this->blacklist->exists()) {

            if ($this->whitelist->exists()) {
                $this->whitelist->read();
            }
            if ($this->blacklist->exists()) {
                $this->blacklist->read();
            }

            if ($this->blacklist->isBlocked() && ! ($this->whitelist->exists() && $this->whitelist->isBlocked())) {
                /* access denied */
                $YANA->report( new Error('block'));
                $YANA->exitTo(null); /* die */
                return false;
            } else {
                /* access granted */
                return true;
            }

        } else {
            return true;
        }
    }

}
// Copyright  2009  Thomas Meyer  http://www.yanaframework.net
?>