<?php
/**
 * Plugin for Yana PHP-Framework
 *
 * Software:        Yana PHP-Framework 3.0.0
 * Author:          Dipl. Inf. Thomas Meyer
 * Homepage:        http://www.yanaframework.net
 * Copyright:       (c) 2009  Thomas Meyer
 * License:         GNU GPL  http://www.gnu.org/licenses/
 *
 * This program: can be redistributed and/or modified under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/.
 *
 * You may use this software for free. In case you like it, or want to
 * use it for a commercial purpose, please consider making a donation.
 * http://yanaframework.net/donate.php
 *
 * This notice MAY NOT be removed.
 *
 *
 * @author     Thomas Meyer
 * @license    http://www.gnu.org/licenses/gpl.txt
 * @package    yana
 * @subpackage plugins
 */

/**
 * RSS to HTML factory setup
 *
 * This implements basic setup functions for
 * the RSS to HTML factory plugin
 *
 * @access     public
 * @package    yana
 * @subpackage plugins
 */
class plugin_rss_admin extends Plugin
{
    /**
     * @access private
     * @var    SML
     * @ignore
     */
    var $file;

    /**
     * Constructor
     *
     * @param string $name a name for this plugin
     * @access public
     * @ignore
     */
    function plugin_rss_admin($name)
    {
        settype($name,"string");
        global $YANA;
        $this->name = $name;
        $this->file = $YANA->registry->files['config'];
    }

    /**
     * Default event handler
     *
     * returns bool(true) on success and bool(false) on error
     *
     * @access  public
     * @return  bool
     * @param   string  $event  name of the called event in lower-case
     * @param   array   $ARGS   array of arguments passed to the function
     * @ignore
     */
    function _default($event, $ARGS)
    {
        global $YANA;
        /* do nothing */
        return true;
    }

    /**
     * create configuration panel
     *
     * parameters taken:
     *
     * <ul>
     * <li> string id </li>
     * </ul>
     *
     * @access  public
     * @param   array  $ARGS  ignored
     * @return  bool
     * @ignore
     */
    function get_rss_to_html_config($ARGS)
    {
        settype($ARGS, "array");
        global $YANA;
        $YANA->setVar("ID",((!empty($ARGS['id'])) ? $ARGS['id'] : ""));
        return true;
    }

    /**
     * Save configuration data to profi
     *
     * parameters taken:
     *
     * <ul>
     * <li> string rss/file </li>
     * </ul>
     *
     * @access  public
     * @param   array  $ARGS  ignored
     * @return  bool
     * @ignore
     */
    function set_rss_to_html_config($ARGS)
    {
        settype($ARGS, "array");
        global $YANA;
        if ($ARGS['rss/file'] && !file_exists($ARGS['rss/file'])) {
            return false;
        } else if (empty($ARGS['id'])) {
            return $YANA->handle('set_config_default', $ARGS);
        } else {
            return $YANA->handle('set_config_profile', $ARGS);
        }
    }

}

//  2006, Thomas Meyer www.All-Community.de
// Copyright  2009  Thomas Meyer  http://www.yanaframework.net
?>