<?php
/**
 * Plugin for Yana PHP-Framework
 *
 * Software:        Yana PHP-Framework 3.0.0
 * Author:          Dipl. Inf. Thomas Meyer
 * Homepage:        http://www.yanaframework.net
 * Copyright:       (c) 2009  Thomas Meyer
 * License:         GNU GPL  http://www.gnu.org/licenses/
 *
 * This program: can be redistributed and/or modified under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/.
 *
 * You may use this software for free. In case you like it, or want to
 * use it for a commercial purpose, please consider making a donation.
 * http://yanaframework.net/donate.php
 *
 * This notice MAY NOT be removed.
 *
 *
 * @author     Thomas Meyer
 * @license    http://www.gnu.org/licenses/gpl.txt
 * @package    yana
 * @subpackage plugins
 */

/**
 * Search plugin
 *
 * This plugin searches an index for words.
 *
 * @access     public
 * @package    yana
 * @subpackage plugins
 */
class plugin_search extends Plugin
{
    /**#@+
     * @ignore
     * @access  private
     */

    /** @var string */ var $search_string = "";
    /** @var array  */ var $cache = array();

    /**#@-*/

    /**
     * Default event handler
     *
     * returns bool(true) on success and bool(false) on error
     *
     * @access  public
     * @return  bool
     * @param   string  $event  name of the called event in lower-case
     * @param   array   $ARGS   array of arguments passed to the function
     * @ignore
     */
    function _default($event, $ARGS)
    {
        global $YANA;
        /* do nothing */
        return true;
    }

    /**
     * Show table of statistics
     *
     * this function does not expect any arguments
     *
     * Type: read
     * Permission: 0
     * Templates:
     *
     * @access  public
     * @param   array  $ARGS  array of params passed to the function
     * @return  bool
     */
    function search_stats ($ARGS)
    {
        settype($ARGS, "array");
        global $YANA;

        if (class_exists('Microsummary')) {
            Microsummary::publish($this->name);
        }

        $statistik = new counter($this->name);
        $zahlen    = $statistik->get();
        if (!empty($zahlen)) {
            uasort($zahlen, array($this, "_sort"));
            $YANA->setVar("STATS", $zahlen);
        }

        return true;
    }

    /**
     * Search index for a specific string
     *
     * Type: read
     * Permission: 0
     * Templates:
     *
     * parameters taken:
     *
     * <ul>
     * <li> string target string to search for </li>
     * </ul>
     *
     * @access  public
     * @param   array  $ARGS  array of params passed to the function
     * @return  bool
     */
    function search_start ($ARGS)
    {
        global $YANA;
        settype($ARGS['target'], "string");

        $this->search_string = preg_replace("/[^\s\w\d&;]/i", "", $ARGS['target']);

        if (empty($this->search_string)) {
            return true;
        }

        $statistik = new counter($this->name);
        assert('!isset($temp);');
        $temp = explode(" ", strtolower($this->search_string));
        for ($i = 0; $i < count($temp); $i++)
        {
            /* update counter value */
            assert('!isset($counter_id);');
            assert('!isset($counter_info);');
            assert('!isset($counter_value);');
            $dummy = null;
            $counter_id = $this->_encode($temp[$i], $dummy);
            unset($dummy);
            $counter_info = $temp[$i];
            $counter_value = $statistik->count($counter_id,$counter_info);
            /* update Microsummary */
            if (class_exists('Microsummary')) {
                assert('!isset($zahlen);');
                $zahlen = $statistik->getCount();
                if (is_array($zahlen) && count($zahlen) > 0) {
                    $most_wanted = array_pop($zahlen);
                    if ($most_wanted <= $counter_value) {
                        Microsummary::set($this->name, strtoupper($this->name).' most wanted: '.$counter_info.'('.$counter_value.')');
                    }
                }
                unset($zahlen);
            }
            unset($counter_id,$counter_info,$counter_value);
        } /* end for */
        unset($temp);

        $hitlist = $this->_from_cache($this->search_string);
        if (empty($hitlist))
        $hitlist = $this->_commit($this->search_string);

        $YANA->setVar("SUBJECT", $this->search_string);
        $target = $YANA->getVar("PROFILE.SEARCH.TARGET");
        if (empty($target)) {
            $target = "_self";
        }
        $prefix = $YANA->getVar("PROFILE.SEARCH.PREFIX");

        assert('!isset($results); // Cannot redeclare var $results');
        $results = array();
        for ($i = 0; $i < count($hitlist); $i++)
        {
            if (count($hitlist[$i]) > 0 && isset($hitlist[$i][0]) && $hitlist[$i][0] != "\n") {
                $url = $prefix . htmlentities(preg_replace("/^.\//", "", preg_replace("/^\n/", "", $hitlist[$i][0])));
                if (!empty($hitlist[$i][1]) || !empty($hitlist[$i][2])) {
                    $results[] = array(
                        'URL' => $url,
                        'TARGET' => $target,
                        'TITLE' => @$hitlist[$i][1],
                        'TEXT' => @$hitlist[$i][2]
                    );
                }
            }
        } /* end for */
        unset($i);

        $YANA->setVar("RESULTS", $results);

        return true;

    }

    /**
     * Create index file for search engine
     *
     * Type: primary
     * Permission: 100
     * Templates:
     *
     * parameters taken:
     *
     * <ul>
     * <li> string  $dir               source directory </li>
     * <li> bool    $recurse  = false  recurse sub-directories </li>
     * <li> bool    $meta     = false  include meta-tags (title, keywords aso.) </li>
     * </ul>
     *
     * @access  public
     * @param   array  $ARGS  array of params passed to the function
     * @return  bool
     */
    function search_create_index ($ARGS)
    {
        settype($ARGS, "array");
        global $YANA;

        if (!headers_sent()) {
            header("Content-type: text/plain");
        }

        @set_time_limit(500);

        /**
         * 1) Check input arguments
         */
         
        /* 1.1) argument 'dir' */
        if (!isset($ARGS['dir'])) {
            print "WARNING: Missing argument 'dir'.\n";
            return false;
        } elseif (!is_string($ARGS['dir'])) {
            print "WARNING: Invalid type for argument 'dir'. String expected.\n";
            return false;
        } elseif (!is_dir($ARGS['dir'])) {
            print "WARNING: Directory '${ARGS['dir']}' not found.\n";
            return false;
        } else {
            $dir = $ARGS['dir'];
        }

        /* 1.2) argument 'recurse' */
        if (empty($ARGS['recurse'])) {
            $recurse = false;
        } elseif ($ARGS['recurse']) {
            $recurse = true;
        } else {
            $recurse = false;
        }

        /* 1.3) argument 'meta' */
        if (empty($ARGS['meta'])) {
            $meta = false;
        } elseif ($ARGS['meta']) {
            $meta = true;
        } else {
            $meta = false;
        }

        /**
         * 2) cache input values for later use
         */
        $YANA->handle('set_config_default', array($this->name => array('dir' => $dir, 'recurse' => $recurse, 'meta' => $meta)));

        /**
         * 3) Open document list for output
         */
        $resultKeywords = array();
        $currentDocument = 0;
        $fKeywords = $YANA->plugins->get($this->name, "KEYWORDS");
        $fKeywords = $fKeywords->getPath();
        $fDocuments = $YANA->plugins->get($this->name, "DOCUMENTS");
        $fDocuments = $fDocuments->getPath();
        if (($hDocuments = fopen($fDocuments, "w+")) === false) {
            print "ERROR: Unable to open index-file '$fDocuments' for output.\n";
            return false;
        }

        /**
         * 4) Scan dir
         */
        assert('!isset($i); // Cannot redeclare var $i');
        assert('!isset($file); // Cannot redeclare var $file');
        foreach (plugin_search::_getListOfFiles($dir, '*.htm|*.html|*.xml|*.shtml', $recurse) as $i =>  $file)
        {
            if (($handle = fopen("$file", "r")) === false) {
                print "NOTICE: Unable to open file '$file'.\n";
                continue;
            } else {
                fwrite($hDocuments, $file);
            }

            print "(${i}) ${file}\n";

            assert('!isset($docTitle); // Cannot redeclare var $docTitle');
            $docTitle = false;
            assert('!isset($docDesc); // Cannot redeclare var $docDesc');
            $docDesc = "";
            assert('!isset($keywords); // Cannot redeclare var $keywords');
            $keywords = false;
            assert('!isset($headContent); // Cannot redeclare var $headContent');
            $headContent = null;

            assert('!isset($content); // Cannot redeclare var $content');
            $content = '';
            assert('!isset($h); // Cannot redeclare var $h');
            while (!feof($handle))
            {
                $content .= utf8_decode(fread($handle, 8192));
                $h = null;
                if ($docTitle === false) {
                    if (preg_match('/<title>\s*([^<]+)\s*<\/title>/Usi', $content, $docTitle) ||
                        preg_match('/<meta\s+name="title"\s+content="([^">]+)"/Usi', $content, $docTitle)) {
                        $content = str_replace($docTitle[0], '', $content);
                        $docTitle = preg_replace('/\s+/s', ' ', $docTitle[1]);
                    } else {
                        $docTitle = false;
                    }
                }
                if ($meta === true && $keywords === false && preg_match('/<head>.*?<\/head>/si', $content, $headContent)) {
                    $headContent = $headContent[0];
                    while (preg_match('/<meta\s+name="keywords"\s+content="([^">]+)"/Usi', $headContent, $h))
                    {
                        $headContent = str_replace($h[0], '', $headContent);
                        $keywords = array();
                        if (strpos($h[1], ',') !== false) {
                            $keywords = array_merge($keywords, explode(',', preg_replace('/\s{2,}/', ' ', $h[1])));
                        } else {
                            $keywords[] = preg_replace('/\s+/s', ' ', $h[1]);
                        }
                    }
                }
                if (empty($docDesc)) {
                    $content = preg_replace('/^.*<body[^>]*>\s*/si', '', $content);
                }
                $content = preg_replace('/<[^>]+>/', ' ', $content);
                $content = html_entity_decode($content);
                $content = strip_tags($content);
                $content = preg_replace('/[\s\t]+/', ' ', $content);
                if (strlen($docDesc) < 150) {
                    $docDesc .= trim($content);
                }
                $content = preg_replace('/[^a-zA-Z\s]+/', ' ', $content);
                $content = preg_replace('/[\s\t]+/', ' ', $content);
                $content = explode(' ', $content);
                for ($i = 0; $i < count($content) - 1; $i++)
                {
                    if ($content[$i] !== '') {
                        if (!isset($resultKeywords[$content[$i]])) {
                            $resultKeywords[$content[$i]] = array(0 => array(), 1 => array($currentDocument));
                        } else {
                            $resultKeywords[$content[$i]][1][] = $currentDocument;
                        }
                    }
                }
                unset($i);
                $content = array_pop($content) . ' ';
            }
            unset($content, $h);

            if (fclose($handle) === false) {
                trigger_error("Unable to close file '$file'.", E_USER_NOTICE);
                continue;
            }

            if (empty($docTitle)) {
                $docTitle = $file;
            } else {
                $content = html_entity_decode($docTitle);
                $content = strip_tags($content);
                $content = explode(' ', $content);
                for ($i = 0; $i < count($content); $i++)
                {
                    if ($content[$i] !== '') {
                        if (!isset($resultKeywords[$content[$i]])) {
                            $resultKeywords[$content[$i]] = array(0 => array(), 1 => array($currentDocument));
                        } else {
                            $resultKeywords[$content[$i]][1][] = $currentDocument;
                        }
                    }
                }
                unset($content, $i);
            }
            fwrite($hDocuments, ',' . str_replace(',', '', $docTitle) . ',' . htmlentities(substr($docDesc, 0, 150)) . "]\n");

            if (is_array($keywords)) {
                for ($i = 0; $i < count($keywords); $i++)
                {
                    if (!isset($resultKeywords[$keywords[$i]])) {
                        $resultKeywords[$keywords[$i]] = array(0 => array(), 1 => array($currentDocument));
                    } else {
                        $resultKeywords[$keywords[$i]][1][] = $currentDocument;
                    }
                }
                unset($i);
            }

            unset($keywords, $headContent, $docTitle, $docDesc);
            
            ++$currentDocument;
        } /* end foreach */
        unset($i, $file); /* clean up garbage */

        /**
         * 5) compress results
         */
        assert('!isset($keyword); // Cannot redeclare var $keyword');
        assert('!isset($array); // Cannot redeclare var $array');
        assert('!isset($compare); // Cannot redeclare var $compare');
        assert('!isset($newKeyword); // Cannot redeclare var $newKeyword');
        foreach ($resultKeywords as $keyword => $array)
        {
            $compare = "";
            $newKeyword = $this->_encode($keyword, $compare);

            if ($newKeyword === '') {
                unset($resultKeywords[$keyword]);
            } elseif ($newKeyword !== $keyword) {
                unset($resultKeywords[$keyword]);
                if (isset($resultKeywords[$newKeyword])) {
                    if ($compare !== '') {
                        if (!isset($resultKeywords[$newKeyword][0])) {
                            $resultKeywords[$newKeyword][0] = array($compare);
                        } elseif (!in_array($compare, $resultKeywords[$newKeyword][0])) {
                            $resultKeywords[$newKeyword][0][] = $compare;
                        }
                    }
                    assert('!isset($id); // Cannot redeclare var $id');
                    foreach ($array[1] as $id)
                    {
                        if (!in_array($id, $resultKeywords[$newKeyword][1])) {
                            $resultKeywords[$newKeyword][1][] = $id;
                        }
                    }
                    unset($id);
                } else {
                    if ($compare !== '') {
                        $resultKeywords[$newKeyword] = array(0 => array($compare), 1 => $array[1]);
                    } else {
                        $resultKeywords[$newKeyword] = array(0 => array(), 1 => $array[1]);
                    }
                }
            }
        }
        unset($keyword, $array, $compare, $newKeyword);
        ksort($resultKeywords);

        /**
         * 6) close document list
         */
        if (fclose($hDocuments) === false) {
            print "ERROR: Unable to close index-file '$fDocuments'.\n";
            return false;
        }

        print "\n#" . count($resultKeywords) . " keywords indexed\n";

        /**
         * 7) output keywords
         */
        if (($hKeywords = fopen($fKeywords, "w+")) === false) {
            print "ERROR: Unable to open index-file '$fKeywords' for output.\n";
            return false;
        }

        assert('!isset($keyword); // Cannot redeclare var $keyword');
        assert('!isset($array); // Cannot redeclare var $array');
        foreach ($resultKeywords as $keyword => $array)
        {
            fwrite($hKeywords, $keyword . '=');
            if (count($array[0]) > 0) {
                fwrite($hKeywords, '|'. implode('|', array_unique($array[0])));
            }
            if (count($array[1]) > 0) {
                fwrite($hKeywords, '"'. implode('"', array_unique($array[1])) . '"');
            }
            unset($resultKeywords[$keyword]);
            if (count($resultKeywords) > 0) {
                fwrite($hKeywords, ', ');
            }
        }
        unset($keyword, $array, $compare, $newKeyword);
        if (fclose($hKeywords) === false) {
            print "ERROR: Unable to close index-file '$fKeywords' for output.\n";
            return false;
        }

        return true;
    }

    /**
     * upload search index files to server
     *
     * returns bool(true) on success and bool(false) on error
     *
     * @access  public
     * @return  bool
     * @param   array   $ARGS   array of arguments passed to the function
     * @ignore
     */
    function search_write_upload($ARGS)
    {
        global $YANA;

        /**
         * 1) get path of target files
         */
        $documents_dat = $YANA->plugins->get($this->name, "DOCUMENTS");
        if (!is_object($documents_dat)) {
            return false;
        } else {
            $documents_dat = $documents_dat->getPath();
        }

        $keywords_dat = $YANA->plugins->get($this->name, "KEYWORDS");
        if (!is_object($keywords_dat)) {
            return false;
        } else {
            $keywords_dat = $keywords_dat->getPath();
        }

        /**
         * 2) check if both files have been provided
         */
        if (empty($_FILES['documents_dat']['tmp_name'])) {
            $YANA->report( new Warning('field_missing', array('FIELD' => '"documents.dat"') ) );
            return false;

        } elseif (empty($_FILES['keywords_dat']['tmp_name'])) {
            $YANA->report( new Warning('field_missing', array('FIELD' => '"keywords.dat"') ) );
            return false;

        /**
         * 3) check names of uploaded files
         */
        } elseif ($_FILES['documents_dat']['name'] !== 'documents.dat') {
            $YANA->report( new Warning('field_invalid', array('FIELD' => 'documents.dat = "' . $_FILES['documents_dat']['name'] . '"') ) );
            return false;

        } elseif ($_FILES['keywords_dat']['name'] !== 'keywords.dat') {
            $YANA->report( new Warning('field_invalid', array('FIELD' => '"keywords.dat = "' . $_FILES['keywords_dat']['name'] . '"') ) );
            return false;

        /**
         * 4) move uploaded files to destination
         */
        } elseif (!move_uploaded_file($_FILES['documents_dat']['tmp_name'], $documents_dat)) {
            $YANA->report( new Warning('field_invalid', array('FIELD' => '"documents.dat"') ) );
            return false;

        } elseif (!move_uploaded_file($_FILES['keywords_dat']['tmp_name'], $keywords_dat)) {
            $YANA->report( new Warning('field_invalid', array('FIELD' => '"keywords.dat"') ) );
            return false;

        } else {
            return true;

        }
    }

    /**
     * @access  private
     * @param   string  $subject
     * @return  array
     */
    function _commit($subject)
    {
        global $YANA;
        $found = false;
        $hits = array();
        $hitlist = array();
        $request = explode(" ", $subject);
        $keywords = $YANA->plugins->get($this->name, "KEYWORDS");
        $documents = $YANA->plugins->get($this->name, "DOCUMENTS");
        $documentList = array();

        if (!$keywords->exists() || !$documents->exists()) {
            $YANA->report( new Error('search_failed') );
            $YANA->exitTo('');
            return false;
        }

        $keywords->read();
        $KEYS = preg_replace("/\n/", "", $keywords->get());
        $KEYS = explode(", ", $KEYS);

        assert('!isset($i); // Cannot redeclare var $i');
        for ($i = 0; $i < count($request); $i++)
        {
            $dummy = null;
            $enc_string = $this->_encode($request[$i], $dummy);
            unset($dummy);
            $found = false;
            $max = count($KEYS)-1;
            $min = 0;
            $n = floor($max /2);
            $prev_n = array(-1,-1);

            while (!$found && $max != $min)
            {
                $temp2 = preg_replace("/=.*/i", "", $KEYS[$n]);
                $temp2 = strcasecmp($temp2, $enc_string);
                if ($temp2 > 0) {
                    $max = $n;
                } else if ($temp2<0) {
                    $min = $n;
                } else {
                    $hits[$request[$i]]=explode("\"", preg_replace("/.*=/i", "", $KEYS[$n]));
                    $whatsRelated = trim(preg_replace("/\|/", " ", preg_replace("/~/", $enc_string, $hits[$request[$i]][0])));
                    $whatsRelated = preg_replace("/\s\s/", " ", $whatsRelated);
                    $this->search_string .= ( $hits[$request[$i]][0] != "" && $request[$i] != $whatsRelated ? '<span class="search_related"><a class="search_related" href="javascript:whatsRelated()" target="_self">%LANGUAGE.RELATED%&nbsp;&quot;' .$request[$i]. '&quot;:</a> ' . $whatsRelated . '</span>' : '' );
                    $found = true;
                } /* end if */

                array_shift($prev_n);
                $prev_n[count($prev_n)] = $n;

                if ($min!=$n) {
                        $n = $min + floor( ($max - $min) /2 );
                } else {
                        $n = $min + ceil( ($max - $min) /2 );
                }

                if ($prev_n[0] == $n || $prev_n[1] == $n) {
                    $hits[$request[$i]][0] = "";
                    break;
                }

            } /* end while */
        } /* end for */
        unset($i);

        if (count($hits) > 0) {
             array_shift($hits[$request[0]]); /* 0th is always empty - so drop it */
             $hitlist=$hits[$request[0]];
        }

        /* BEGIN logical AND */
        assert('!isset($i); // Cannot redeclare var $i');
        for ($i = 1; $i < count($request); $i++)
        {
            $myTemp = array();
            assert('!isset($j); // Cannot redeclare var $j');
            for ($j = 1; $j < count($hits[$request[$i]]); $j++)
            {
                if(in_array($hits[$request[$i]][$j], $hitlist)) {
                    $myTemp[count($myTemp)] = $hits[$request[$i]][$j];
                }
            }
            unset($j);
            $hitlist = $myTemp;
        } /* end for */
        unset($i);
        /* END logical AND */

        /* BEGIN load list if documents */
        $zeile = 0;
        $buffer = "";

        $file = fopen($documents->getPath(), "r");
        flock($file, LOCK_SH);
        while (!feof($file))
        {
            $temp = fgetc($file);
            if ($temp == ']') {
                if (in_array((int) $zeile, $hitlist)) {
                    $documentList[$zeile] = explode(',', $buffer);
                }
                $buffer="";
                $zeile++;
            } else {
                $buffer .= $temp;
            }
        } /* end while */
        flock($file, LOCK_UN);
        fclose($file);
        /* END load list if documents */

        if (count($hitlist) >= 1) {
            /* resolve Ids */
            sort($request);
            $this->_to_cache(implode(" ", $request), $hitlist, $documentList);
            assert('!isset($i); // Cannot redeclare var $i');
            for ($i = 0; $i < count($hitlist); $i++)
            {
                if (isset($hitlist[$i]) && $hitlist[$i] != "") {
                    $hitlist[$i] = $documentList[(int) $hitlist[$i]];
                }
            } /* end for */
            unset($i);
        } /* end if */

        return $hitlist;
    }

    /**
     * @access  private
     * @param   string  $subject
     * @param   array   &$value
     * @param   array   &$documents
     * @return  bool
     */
    function _to_cache($subject, &$value, &$documents)
    {
        global $YANA;

        $id = $YANA->getVar('ID');
        if (empty($id)) {
            $id = "default";
        }
        $cacheFile = $YANA->getVar('TEMPDIR')."$id.cache";
        if (sizeOf($this->cache)>30) {
            array_pop($this->cache);
        }

        $this->_cache_id($subject);
        $text = $subject.'='.rawurlencode($this->search_string).";";

        for ($i = 0; $i < count($value); $i++)
        {
            if (isset($documents[$value[$i]])) {
                $text .= rawurlencode(trim(preg_replace("/\s/s", " ", implode(",", $documents[$value[$i]])))).";";
            }
        }
        unset($i);

        $text .= "\n" . implode("", $this->cache);

        if (! file_exists($cacheFile)) {
            $file = fopen($cacheFile, "w+");
            flock($file, LOCK_EX);
            fwrite($file, $text);
            flock($file, LOCK_UN);
            fclose($file);
        } elseif (is_writeable($cacheFile)) {
            $file = fopen($cacheFile, "w+");
            flock($file, LOCK_EX);
            fwrite($file, $text);
            flock($file, LOCK_UN);
            fclose($file);
        } else {
            return false;
        }

        return true;

    }

    /**
     * @access  private
     * @param   string  &$subject
     */
    function _cache_id(&$subject)
    {
        settype($subject, "string");
        $temp = explode(" ", $subject);
        $dummy = null;
        for ($i = 0; $i < sizeOf($temp); $i++)
        {
            $temp[$i] = $this->_encode($temp[$i], $dummy);
        }
        sort($temp);
        $subject = implode(" ", $temp);
    }

    /**
     * @access  private
     * @param   array  $A
     * @param   array  $B
     * @return  int
     */
    function _sort($A, $B)
    {
        if ($A['COUNT'] < $B['COUNT']) {
            return +1;
        } elseif ($A['COUNT'] > $B['COUNT']) {
            return -1;
        } elseif ($A['INFO']  < $B['INFO']) {
            return -1;
        } elseif ($A['INFO']  > $B['INFO']) {
            return +1;
        } else {
            return 0;
        }
    }

    /**
     * @access  private
     * @param   string  $inputString
     * @param   string  &$compare
     * @return  string
     */
    function _encode($inputString, &$compare)
    {
        $compare = "";

        $inputString = strtolower($inputString);
        $inputString = html_entity_decode($inputString);
        $inputString = str_replace('', 'a', $inputString);
        $inputString = str_replace('', 'u', $inputString);
        $inputString = str_replace('', 'o', $inputString);
        $inputString = str_replace('', 'ss', $inputString);
        $inputString = preg_replace('/[^a-zA-Z\s]/', '', $inputString);

        global $YANA;
        $grammar = $YANA->plugins->getVar($this->name, 'GRAMMAR');

        if (in_array($inputString, $grammar['STOPWORDS'])) {
            $inputString = "";
        } else {
            foreach ($grammar['START_WORD'] as $word)
            {
                if ($inputString != $word) {
                    if (preg_match("/^$word/", $inputString)) {
                        $inputString = preg_replace("/^$word/", "", $inputString, 1);
                        $compare = $word . '~';
                        break;
                    }
                }
            }
            foreach ($grammar['END_WORD'] as $word)
            {
                if ($inputString != $word) {
                    if (preg_match("/$word$/", $inputString)) {
                        $inputString = preg_replace("/$word$/", "", $inputString, 1);
                        if ($compare === "") {
                            $compare = '~' . $word;
                        } else {
                            $compare .= $word;
                        }
                        break;
                    }
                }
            }
            $inputString = preg_replace('/\s/s', '', $inputString);
        }

        $inputString = trim($inputString);
        if (strlen($inputString) < 3 || strlen($inputString) > 20) {
            return "";
        } else {
            return $inputString;
        }
    }

    /**
     * @static
     * @access  private
     * @param   string  $dir
     * @param   string  $filter
     * @param   bool    $recurse
     * @return  array
     */
    function _getListOfFiles($dir, $filter, $recurse)
    {
        assert('is_string($dir); // Wrong argument type argument 1. String expected');
        assert('is_dir($dir); // Invalid argument 1. Directory expected');
        assert('is_string($filter); // Wrong argument type argument 2. String expected');
        assert('is_bool($recurse); // Wrong argument type argument 3. Boolean expected');
        $list = array();

        $dir .= '/';

        /* 1 recurse sub-directories */
        if ($recurse) {
            assert('!isset($subdir); // Cannot redeclare var $subdir');
            foreach (dirlist($dir, "", YANA_GET_DIRS) as $subdir)
            {
                /* ignore directories, which start with an underscore */
                if (strpos($subdir, '_') === 0) {
                    continue;
                } else {
                    $list = array_merge($list, plugin_search::_getListOfFiles($dir . $subdir, $filter, true));
                }
            }
            unset($subdir);
        }

        /* 2 get files in current directory */
        assert('!isset($file); // Cannot redeclare var $subdir');
        foreach (dirlist($dir, $filter, YANA_GET_FILES) as $file)
        {
            $list[] = $dir . $file;
        }
        unset($file);

        return $list;
    }

    /**
     * @access  private
     * @param   string  $subject
     * @return  array|bool(false)
     */
    function _from_cache($subject)
    {
        global $YANA;

        $id = $YANA->getVar('ID');
        if (empty($id)) {
            $id = "default";
        }
        $cacheFile = $YANA->getVar('TEMPDIR')."$id.cache";
        $hitlist = array();

        if (file_exists($cacheFile) && is_readable($cacheFile)) {
            $this->cache = file($cacheFile);
        } else {
            return false;
        }

        $this->_cache_id($subject);
        $temp = "";
        for ($i = 0; $i < sizeOf($this->cache); $i++)
        {
            if (preg_match("/^".$subject."=/i", $this->cache[$i])) {
                $temp = preg_replace("/^".$subject."=/", "", $this->cache[$i]);
                break;
            }
        }
        unset($i);

        if (!empty($temp)) {
            $temp = explode(";", $temp);
            $this->search_string = rawurldecode($temp[0]);
            assert('!isset($i); // Cannot redeclare var $i');
            for ($i = 1; $i < count($temp); $i++)
            {
                $hitlist[] = explode(',', rawurldecode($temp[$i]));
            }
            unset($i);
            return $hitlist;
        } else {
            return false;
        }
    }
}
// Copyright  2009  Thomas Meyer  http://www.yanaframework.net
?>