<?php
/**
 * Plugin for Yana PHP-Framework
 *
 * Software:        Yana PHP-Framework 3.0.0
 * Author:          Dipl. Inf. Thomas Meyer
 * Homepage:        http://www.yanaframework.net
 * Copyright:       (c) 2009  Thomas Meyer
 * License:         GNU GPL  http://www.gnu.org/licenses/
 *
 * This program: can be redistributed and/or modified under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/.
 *
 * You may use this software for free. In case you like it, or want to
 * use it for a commercial purpose, please consider making a donation.
 * http://yanaframework.net/donate.php
 *
 * This notice MAY NOT be removed.
 *
 *
 * @author     Thomas Meyer
 * @license    http://www.gnu.org/licenses/gpl.txt
 * @package    yana
 * @subpackage plugins
 */

/**
 * user authentification plugin
 *
 * @access     public
 * @package    yana
 * @subpackage plugins
 */
class plugin_user extends Plugin
{
    /**
     * count boundary
     *
     * Maximum number of times a user may enter
     * a wrong password before its account
     * is suspended for $maxFailureTime seconds.
     *
     * @access  private
     * @var     int
     */
    var $maxFailureCount = 3;
    /**
     * time boundary
     *
     * Maximum time in seconds a user's login
     * is blocked after entering a wrong password
     * $maxFailureCount times.
     *
     * @access  private
     * @var     int
     */
    var $maxFailureTime = 300; /* 300 sec. = 5 minutes */
    /**
     * @access  private
     * @var     string
     */
    var $user_name = "";

    /**
     * constructor
     *
     * @param  string  $name
     * @ignore
     */
    function plugin_user($name)
    {
        global $YANA;
        settype($name, "string");
        $this->name = $name;

        if (!empty($_SESSION['user_name'])) {
            $this->user_name = $_SESSION['user_name'];
            $YANA->setVar("SESSION_USER_ID", $this->user_name);
        } else {
            $YANA->unsetVar("SESSION_USER_ID", $this->user_name);
        }
        $YANA->setVar("SESSION_ID", session_id());
        $YANA->setVar("SESSION_NAME", session_name());
    }

    /**
     * Default event handler
     *
     * returns bool(true) on success and bool(false) on error
     *
     * @access  public
     * @return  bool
     * @param   string  $event  name of the called event in lower-case
     * @param   array   $ARGS   array of arguments passed to the function
     * @ignore
     */
    function _default($event, $ARGS)
    {
        settype($event, "string");
        settype($ARGS, "array");

        global $YANA;
        $YANA->language->read("user");
        if (!empty($ARGS['id'])) {
            $profileId = $ARGS['id'];
        } else {
            $profileId = "";
        }

        if ($this->_checkPermission($this->user_name, $profileId, $event)) {
            /* access granted */
            $YANA->setVar("PERMISSION", $YANA->session->getUserPermission($this->user_name, $profileId));
            return true;
        } else {
            return false;
        }
    }

    /**
     * Set new password
     *
     * @access  public
     * @param   array    $ARGS
     * @return  bool
     */
    function set_pwd($ARGS)
    {
        settype($ARGS, "array");
        global $YANA;

        if (isset($ARGS['pwt_neu']) && isset($ARGS['pwt_wdh']) && $ARGS['pwt_neu']===$ARGS['pwt_wdh'] && is_string($ARGS['pwt_neu']) && is_string($ARGS['pwt_alt'])) {

            if (!$YANA->session->checkLogin($this->user_name,$ARGS['pwt_alt'])) {
                $YANA->report( new Error('LOGIN_INVALID'));
                $YANA->exitTo('');
                return false;
            } else if (!$YANA->session->changePwd($this->user_name,$ARGS['pwt_neu'])) {
                $YANA->report( new Error('500'));
                $YANA->exitTo('index');
                return false;
            } else {
                $YANA->report( new Message('200'));
                $YANA->exitTo('index');
                return true;
            }
        } else {
            return false;
        }

    }

    /**
     * Logout
     *
     * @access  public
     * @param   array    $ARGS
     * @return  bool
     */
    function logout($ARGS)
    {
        global $YANA;
        $YANA->session->logout();
        return true;
    }

    /**
     * Check if login data is correct
     *
     * @access  public
     * @param   array    $ARGS
     * @return  bool
     */
    function check_login($ARGS)
    {
        if (!is_string($ARGS['nutzername']) || !is_string($ARGS['passwort'])) {
            return false;
        } else {
            $userId = $ARGS['nutzername'];
            $userPwd = $ARGS['passwort'];
        }
        if (isset($_SESSION['on_login_goto'])) {
            $nextAction = $_SESSION['on_login_goto'];
            unset($_SESSION['on_login_goto']);
        } else {
            $nextAction = null;
        }
        global $YANA;

        /* initialize user data */
        assert('!isset($user); // Cannot redeclare $user');
        $user = $YANA->session->getUser($userId);

        /* initialize update data */
        assert('!isset($update); // Cannot redeclare $update');
        $update = array();
        if (isset($user['USER_FAILURE_COUNT'])) {
            $update['USER_FAILURE_COUNT'] = (int) $user['USER_FAILURE_COUNT'];
        } else {
            $update['USER_FAILURE_COUNT'] = 0;
        }
        if (isset($user['USER_FAILURE_TIME'])) {
            $update['USER_FAILURE_TIME'] = (int) $user['USER_FAILURE_TIME'];
        } else {
            $update['USER_FAILURE_TIME'] = 0;
        }
        if ($update['USER_FAILURE_TIME'] < time() - $this->maxFailureTime) {
            $update['USER_FAILURE_COUNT'] = 0;
            $update['USER_FAILURE_TIME'] = 0;
        }

        /* 1. exit if user is unknown */
        if (!$YANA->session->userExists($userId)) {
            /* delay output if attempt failed to make brute-force attacks more difficult to commit */
            sleep(2);
            return false;

        /* 2. exit if the user has 3 times tried to login with a wrong password in last 5 minutes */
        } else if ($update['USER_FAILURE_COUNT'] >= $this->maxFailureCount) {
            $YANA->report( new Alert('USER_LOGIN_BLOCKED'));
            return false;

        /* 3. proceed if login is okay */
        } else if ($YANA->session->login($userId, $userPwd)) {
            /* update user information */
            if (isset($user['COUNT_LOGINS'])) {
                $update['COUNT_LOGINS'] = (int) $user['COUNT_LOGINS'];
                $update['COUNT_LOGINS']++;
            } else {
                $update['COUNT_LOGINS'] = 1;
            }
            $update['LAST_LOGIN'] = time();
            $update['USER_FAILURE_TIME'] = 0;
            $update['USER_FAILURE_COUNT'] = 0;
            $YANA->session->insertUser($userId, $update);

            /* route next action */
            if (isset($nextAction) && !empty($nextAction)) {
                $YANA->report( new Message('login_okay'));
                $YANA->exitTo($nextAction);
            }
            $this->user_name = $userId;
            return true;

        /* 4. proceed if login is okay */
        } else {
            /* delay output if attempt failed to make brute-force attacks more difficult to commit */
            if (isset($_SESSION['on_login_goto'])) {
                unset($_SESSION['on_login_goto']);
            }

            /* create a log for each failed login attempt */
            if ($YANA->session->userExists($userId)) {
                $YANA->report(new Log("Login attempt failed for user '".strtolower($userId)."'. Invalid password."));
                if ($update['USER_FAILURE_COUNT'] < $this->maxFailureCount) {
                    $update['USER_FAILURE_COUNT']++;
                }
                $update['USER_FAILURE_TIME'] = time();
                $YANA->session->insertUser($userId, $update);
            }

            sleep(2);
            return false;
        }
    }


    /**
     * _checkPermission
     *
     * @access  private
     * @param   string    $user_name
     * @param   string    $id
     * @param   string    $event
     * @return  bool
     * @ignore
     */
    function _checkPermission($user_name, $id, $event)
    {

        settype($user_name, "string");
        settype($id, "string");
        settype($event, "string");
        global $YANA;
        $user_name = strtoupper($user_name);

        $required_perm = $YANA->plugins->getVar("*", "INTERFACE.$event.PERMISSION");
        $user_perm     = $YANA->session->getUserPermission($user_name, $id);
        if (is_null($required_perm)) {
            $required_perm = $YANA->getDefault('EVENT.PERMISSION');
        }
        settype($required_perm, "integer");
        settype($user_perm, "integer");

        if ($required_perm < 0) {
            $YANA->report(new Error("404"));
            return false;
        }

        if ($required_perm == 0 && $user_perm > -1) { // if no permission is required, then don't care for it
            return true;
        } else {
            if (!$YANA->session->isLoggedIn()) {
                  $meldung = "403";
                  $nextAction = "login";
            } else {
                if ($required_perm <= $user_perm) {
                    return true;
                } else {
                    $meldung = "INSUFFICIENT_RIGHTS";
                    $nextAction = "";
                }
            }
            $_SESSION['on_login_goto'] = $event;
            $YANA->report( new Alert($meldung));
            $YANA->exitTo($nextAction);
            return false;
        }
    }

}
// Copyright  2009  Thomas Meyer  http://www.yanaframework.net
?>