<?php
/**
 * Plugin for Yana PHP-Framework
 *
 * Software:        Yana PHP-Framework 3.0.0
 * Author:          Dipl. Inf. Thomas Meyer
 * Homepage:        http://www.yanaframework.net
 * Copyright:       (c) 2009  Thomas Meyer
 * License:         GNU GPL  http://www.gnu.org/licenses/
 *
 * This program: can be redistributed and/or modified under the
 * terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/.
 *
 * You may use this software for free. In case you like it, or want to
 * use it for a commercial purpose, please consider making a donation.
 * http://yanaframework.net/donate.php
 *
 * This notice MAY NOT be removed.
 *
 *
 * @author     Thomas Meyer
 * @license    http://www.gnu.org/licenses/gpl.txt
 * @package    yana
 * @subpackage plugins
 */

/**
 * user management plugin
 *
 * This creates forms and implements functions to
 * manage user data.
 *
 * @access     public
 * @package    yana
 * @subpackage plugins
 */
class plugin_user_admin extends Plugin
{

    /**
     * @access  private
     * @var     user_admin
     */
    var $file;

    /**
     * @access  private
     * @var     array
     */
    var $visibleColumns = array(
        'get_user_list' => array('user_id', 'user_mail', 'group_id', 'active', 'user_image_active')
    );

    /**
     * Constructor
     *
     * @param string $name a name for this plugin
     * @access public
     * @ignore
     */
    function plugin_user_admin($name)
    {
        settype($name, "string");
        $this->name = $name;
        $this->file = new user_admin();
    }

    /**
     * Default event handler
     *
     * returns bool(true) on success and bool(false) on error
     *
     * @access  public
     * @return  bool
     * @param   string  $event  name of the called event in lower-case
     * @param   array   $ARGS   array of arguments passed to the function
     * @ignore
     */
    function _default($event, $ARGS)
    {
        settype($event, "string");
        settype($ARGS, "array");
        global $YANA;

        if (!$this->file->isEmpty()) {

            switch ($event) {

                case "set_user_pwd":
                    $YANA->language->read("user");
                    if ($this->file->set_user_pwd($ARGS['target'])) {
                        return true;
                    } else {
                        return false;
                    }
                break;

                /* SESSION-ERZEUGEN: Mail-Authentifizierung */
                case "set_user_mail":
                    if (!$YANA->getVar("PROFILE.USER.ALLOW_CREATE")) {
                       $YANA->report( new Error("INPUT_INVALID"));
                       return false;
                    }

                    $db = $YANA->connect("user");
                    $mail = untaintInput(strtolower($ARGS['mail']), 'mail', 255);
                    $name = strtolower($ARGS['username']);

                    if (empty($name) || empty($mail)) {
                        $YANA->report( new Warning("INPUT_INVALID"));
                        return false;
                    }

                    $key = md5($mail);

                    if ($db->exists("user.$name")) {
                        $YANA->report( new Warning("USER_ALREADY_EXISTS"));
                        return false;
                    }

                    /**
                     * 1) Remove duplicates
                     *
                     * Find and delete all previous entries of users with the same mail address
                     */
                    $duplicates = $db->get("newuser.*.newuser_id", array(array("newuser_key", $key, '=')));
                    if (is_array($duplicates)) {
                       foreach ($duplicates as $newuser_id)
                       {
                           $db->remove("newuser.$newuser_id");
                       }
                    }
                    unset($duplicates);
                    $db->write();

                    /**
                     * 2) Remove timed out entries
                     *
                     * limit = 24h = 86400sek
                     */
                    $old_entries = $db->get("newuser");
                    $limit = time() - 86400;
                    if (is_array($old_entries)) {
                        foreach ($old_entries as $row)
                        {
                            if ($row['NEWUSER_UTC'] < $limit) {
                                $db->remove("newuser.".$row['NEWUSER_ID']);
                            }
                        }
                    }
                    unset($old_entries);
                    $db->write();

                    /**
                     * 4) add row to table
                     */
                    if (!$db->insert("newuser.*", array("newuser_name" => $name, "newuser_key" => $key, "newuser_mail" => $mail))) {
                        $YANA->report( new Warning("INPUT_INVALID"));
                        return false;
                    } elseif (!$db->write()) {
                        return false;
                    }

                    $YANA->setVar('WEBSITE_URL', $YANA->getVar("REFERER"));
                    $YANA->setVar('KEY', $key);
                    $YANA->setVar('MAIL', $mail);
                    user_admin::sendMail($mail, $YANA->skin->getFile("USER_CONFIRM_MAIL"));

                    return true;
                break;

                case "get_profile_edit":
                    $YANA->setVar("DESCRIPTION", $YANA->getVar("LANGUAGE.DESCR_USER_EDIT"));
                    return $this->file->get_user($_SESSION['user_name'], true);
                break;

                case "set_profile_edit":
                    return $this->file->set_profile_edit($ARGS);
                break;

                default:
                    $YANA->language->read("user");
                break;

            } /* end switch */
        }
        return true;
    }

    /**
     * Form to create a new user
     *
     * Note: visitors must be allowed to sign on
     * for themselves, or this will fail.
     *
     * Shows a form, where visitors may enter
     * the prefered user name and mail address.
     *
     * A mail with the login link will be send
     * to them. (implements double-opt-in
     * procedure)
     *
     * @access  public
     * @param   array    $ARGS
     * @return  bool
     */
    function get_user_mail($ARGS)
    {
        global $YANA;

        if ($this->file->isEmpty()) {
            return false;
        } elseif (!$YANA->getVar("PROFILE.USER.ALLOW_CREATE")) {
            $YANA->report( new Error('404'));
            $YANA->exitTo('');
            return false;
        } else {
            return true;
        }
    }

    /**
     * Create a new user
     *
     * This function registers new user.
     * Note: visitors must be allowed to sign on
     * for themselves, or this will fail.
     *
     * This checks the provided authentification
     * id and if valid, creates a new user
     * according to the dataset in table "newuser".
     * Settings of this new user depend on the
     * system settings for the default user
     * (see file: system.config).
     *
     * @access  public
     * @param   array    $ARGS
     * @return  bool
     */
    function user_authentification($ARGS)
    {
        global $YANA;

        if ($this->file->isEmpty()) {
            return false;
        } else if (!$YANA->getVar("PROFILE.USER.ALLOW_CREATE")) {
            $YANA->report( new Error("INPUT_INVALID"));
            return false;
        } else {

            $db = $YANA->connect("user");

            if (!$db->exists('newuser', array(array('newuser_key', $ARGS['target'], '=')))) {
                $YANA->report( new Error("INPUT_INVALID"));
                return false;
            } else {
                $dbQuery = new DbQuery($db);
                $dbQuery->setType(YANA_DB_SELECT);
                $dbQuery->setTable("newuser");
                $dbQuery->setWhere(array(array('newuser_key', $ARGS['target'],'=')));
                $dbQuery->setLimit(1);
                $entry = $db->get($dbQuery);
                assert('is_array($entry); // Array expected: $entry. Invalid dataset or invalid query.');
                if (empty($entry)) {
                    $YANA->report( new Error("500"));
                    return false;
                } else {
                    $entry = array_pop($entry);
                }
                if (!isset($entry['NEWUSER_NAME']) || !isset($entry['NEWUSER_MAIL'])) {
                    $YANA->report( new Error("500"));
                    return false;
                }
                if ($this->file->user_authentification($entry['NEWUSER_NAME'], $entry['NEWUSER_MAIL'])) {
                    $YANA->report( new Alert('FIRST_LOGIN'));
                    $YANA->exitTo('');
                    return true;
                } else {
                    return false;
                }
            } /* end if */
        } /* end if */
    }

    /**
     * Create "new user" form
     *
     * this action expects no arguments
     *
     * on success:
     * ->  user::set_user_new
     *
     * @access  public
     * @param   array    $ARGS
     * @return  bool
     */
    function get_user_new($ARGS)
    {
        return true;
    }

    /**
     * user configuration panel
     *
     * @access  public
     * @param   array    $ARGS
     * @return  bool
     */
    function get_user_list($ARGS)
    {
        global $YANA;
        $YANA->setVar("DESCRIPTION", $YANA->getVar("LANGUAGE.DESCR_USER_CONFIGURATION"));
        $YANA->setVar("VISIBLE_COLUMNS", $this->visibleColumns[__FUNCTION__]);
        return true;
    }

    /**
     * user configuration panel
     *
     * @access  public
     * @param   array    $ARGS
     * @return  bool
     */
    function get_access_list($ARGS)
    {
        global $YANA;
        $YANA->setVar("DESCRIPTION", $YANA->getVar("LANGUAGE.DESCR_USER_CONFIGURATION"));
        if (isset($ARGS['target']) && isset($ARGS['target']['user_id'])) {
            $YANA->setVar("WHERE", array('user_id', $ARGS['target']['user_id'], '='));
            $_SESSION['useraccess'][__FUNCTION__] = $ARGS['target']['user_id'];
        } else if (isset($_SESSION['useraccess'][__FUNCTION__])) {
            $YANA->setVar("WHERE", array('user_id', $_SESSION['useraccess'][__FUNCTION__], '='));
        }
        return true;
    }

    /**
     * view user profile
     *
     * @access  public
     * @param   array    $ARGS
     * @return  bool
     */
    function view_profile($ARGS)
    {
        if (isset($ARGS['target']) && isset($ARGS['target']['user_id'])) {
            $userId = $ARGS['target']['user_id'];
            $_SESSION['user'][__FUNCTION__] = $ARGS['target']['user_id'];
        } else if (isset($_SESSION['user'][__FUNCTION__])) {
            $userId = $_SESSION['user'][__FUNCTION__];
        } else {
            return false;
        }
        return $this->file->get_user($userId);
    }

    /**
     * Delete user
     *
     * @access  public
     * @param   array    $ARGS
     * @return  bool
     */
    function set_user_edit($ARGS)
    {
        global $YANA;

        if (!$this->file->isEmpty()) {

            $structure = new DbStructure('user');
            $updatedEntries = FormCreator::getFormdata(FORM_EDIT, $structure, 'user');

            /* no data has been provided */
            if (empty($updatedEntries)) {
                $YANA->report(new Warning('INPUT_INVALID'));
                return false;
            }

            foreach ($updatedEntries as $id => $entry)
            {
                $id = strtolower($id);

                foreach (array_keys($entry) as $i)
                {
                    if (!in_array($i, $this->visibleColumns['get_user_list'])) {
                        unset($entry[$i]);
                    }
                }

                /* before doing anything, check if entry exists */
                if (!$YANA->session->userExists($id)) {

                    /* error - no such entry */
                    $YANA->report(new Warning('INPUT_INVALID'));
                    return false;

                /* update the row */
                } else if (!$this->file->set_user_edit($id, $entry)) {
                    /* error - unable to perform update - possibly readonly */
                    return false;
                }
            } /* end for */

            /* commit changes */
            return true;

        } else {

            $YANA->report(new Warning('INPUT_INVALID'));
            return false;

        }

    }

    /**
     * show image
     *
     * @access  public
     * @param   array    $ARGS
     * @return  bool
     */
    function get_profile_image($ARGS)
    {
        if (!empty($ARGS['thumb'])) {
            return $this->file->get_profile_image($ARGS['target'], true);
        } else {
            return $this->file->get_profile_image($ARGS['target'], false);
        }
    }

    /**
     * Delete user
     *
     * @access  public
     * @param   array    $ARGS
     * @return  bool
     */
    function set_user_delete($ARGS)
    {
        global $YANA;

        if (!$this->file->isEmpty()) {

            /* check if user forgot to mark at least 1 row */
            if (empty($ARGS['selected_entries']) || !is_array($ARGS['selected_entries'])) {
                $YANA->report(new Warning('INPUT_INVALID'));
                return false;
            } else {
                /* remove entry from database */
                foreach ($ARGS['selected_entries'] as $id)
                {
                    if (!$this->file->set_user_delete($id)) {
                        return false;
                    }
                } /* end for */
                return true;
            }

        } else {

            $YANA->report(new Warning('INPUT_INVALID'));
            return false;

        }

    }

    /**
     * Insert a new user
     *
     * handles input from action "get_user_new"
     * this action expects no arguments
     *
     * @access  public
     * @param   array    $ARGS
     * @return  bool
     */
    function set_user_new($ARGS)
    {
        global $YANA;
        $YANA->setVar('ID', $_POST['id']);
        return $this->file->set_user_new();
    }

    /**
     * set_access_edit
     *
     * returns bool(true) on success and bool(false) on error
     *
     * @access  public
     * @return  bool
     * @param   array  $ARGS  array of params passed to the function
     */
    function set_access_edit ($ARGS)
    {
        /* check input data */
        assert('is_array($ARGS);');
        settype($ARGS, "array");

        /* global variables */
        global $YANA;
        $database = $YANA->connect('user');

        $updatedEntries = FormCreator::getFormdata(FORM_EDIT, $database->structure, 'useraccess');

        /* no data has been provided */
        if (empty($updatedEntries)) {
            $YANA->report(new Warning('INPUT_INVALID'));
            return false;
        }

        foreach ($updatedEntries as $id => $entry)
        {
            $id = strtolower($id);

            /* before doing anything, check if entry exists */
            if (!$database->exists("useraccess.${id}")) {

                /* error - no such entry */
                $YANA->report(new Warning('INPUT_INVALID'));
                return false;

            /* update the row */
            } else if (!$database->update("useraccess.${id}", $entry)) {
                /* error - unable to perform update - possibly readonly */
                return false;
            }
        } /* end for */
        /* commit changes */
        return $database->write();
    }

    /**
     * set_access_delete
     *
     * returns bool(true) on success and bool(false) on error
     *
     * @access  public
     * @return  bool
     * @param   array  $ARGS  array of params passed to the function
     */
    function set_access_delete ($ARGS)
    {
        /* check input data */
        assert('is_array($ARGS);');
        settype($ARGS, "array");

        /* global variables */
        global $YANA;
        $database = $YANA->connect('user');

        /* check if user forgot to mark at least 1 row */
        if (empty($ARGS['selected_entries']) || !is_array($ARGS['selected_entries'])) {
            $YANA->report(new Warning('INPUT_INVALID'));
            return false;
        }
        /* remove entry from database */
        foreach ($ARGS['selected_entries'] as $id)
        {
            if (!$database->remove("useraccess.${id}")) {
                /* entry does not exist */
                $YANA->report(new Warning('INPUT_INVALID'));
                return false;
            }
        } /* end for */
        /* commit changes */
        return $database->write();
    }

    /**
     * set_access_new
     *
     * returns bool(true) on success and bool(false) on error
     *
     * @access  public
     * @return  bool
     * @param   array  $ARGS  array of params passed to the function
     */
    function set_access_new ($ARGS)
    {
        /* check input data */
        assert('is_array($ARGS);');
        settype($ARGS, "array");

        /* global variables */
        global $YANA;
        $database = $YANA->connect('user');

        $newEntry = FormCreator::getFormdata(FORM_NEW, $database->structure, 'useraccess');

        /* no data has been provided */
        if (empty($newEntry)) {
            $YANA->report(new Warning('INPUT_INVALID'));
            return false;
        }

        /* insert new entry into table */
        if (!$database->insert("useraccess.*", $newEntry)) {
            $YANA->report(new Warning('INPUT_INVALID'));
            return false;
        } else {
            return $database->write();
        }
    }

}
// Copyright  2009  Thomas Meyer  http://www.yanaframework.net
?>